@@ -217,18 +217,16 @@ struct ntp_control {
 #define	CP_FILTERROR	34
 #define	CP_FLASH	35
 #define CP_TTL		36
-#define CP_TTLMAX	37
-#define CP_VARLIST	38
+#define CP_VARLIST	37
 #ifdef OPENSSL
-#define CP_FLAGS	39
-#define CP_HOST		40
-#define	CP_CERTIF	41
-#define	CP_SESKEY	42
-#define	CP_INITSEQ	43
-#define	CP_INITKEY	44
-#define	CP_INITTSP	45
-#define	CP_DIGEST	46
-#define	CP_MAXCODE	CP_DIGEST
+#define CP_FLAGS	38
+#define CP_HOST		39
+#define	CP_INITSEQ	40
+#define	CP_INITKEY	41
+#define	CP_INITTSP	42
+#define	CP_DIGEST	43
+#define CP_IDENT	44
+#define	CP_MAXCODE	CP_IDENT
 #else
 #define	CP_MAXCODE	CP_VARLIST
 #endif /* OPENSSL */
@@ -257,7 +255,7 @@ struct ntp_control {
  * ntp_request.c wants to see this.
  */
 struct ctl_trap {
-	struct sockaddr_in tr_addr;	/* address of trap recipient */
+	struct sockaddr_storage tr_addr;/* address of trap recipient */
 	struct interface *tr_localaddr;	/* interface to send this through */
 	u_long tr_settime;		/* time trap was set */
 	u_long tr_count;		/* async messages sent to this guy */
