@@ -2,6 +2,8 @@
  * ntp_stdlib.h - Prototypes for NTP lib.
  */
 #include <sys/types.h>
+#include <sys/socket.h>
+#include "ntp_rfc2553.h"
 
 #include "ntp_types.h"
 #include "ntp_string.h"
@@ -63,6 +65,7 @@ extern	int	ymd2yd		P((int, int, int));
 extern	int	MD5authdecrypt	P((u_char *, u_int32 *, int, int));
 extern	int	MD5authencrypt	P((u_char *, u_int32 *, int));
 extern	void	MD5auth_setkey	P((keyid_t, const u_char *, const int));
+extern	u_int32	addr2refid	P((struct sockaddr_storage *));
 
 extern	int	atoint		P((const char *, long *));
 extern	int	atouint		P((const char *, u_long *));
@@ -70,8 +73,8 @@ extern	int	hextoint	P((const char *, u_long *));
 extern	char *	humandate	P((u_long));
 extern	char *	humanlogtime	P((void));
 extern	char *	inttoa		P((long));
-extern	char *	mfptoa		P((u_long, u_long, int));
-extern	char *	mfptoms		P((u_long, u_long, int));
+extern	char *	mfptoa		P((u_long, u_long, short));
+extern	char *	mfptoms		P((u_long, u_long, short));
 extern	const char * modetoa	P((int));
 extern  const char * eventstr   P((int));
 extern  const char * ceventstr  P((int));
@@ -79,16 +82,18 @@ extern	char *	statustoa	P((int, int));
 extern  const char * sysstatstr P((int));
 extern  const char * peerstatstr P((int));
 extern  const char * clockstatstr P((int));
-extern	u_int32	netof		P((u_int32));
+extern	struct sockaddr_storage* netof P((struct sockaddr_storage*));
 extern	char *	numtoa		P((u_int32));
 extern	char *	numtohost	P((u_int32));
+extern char * socktoa           P((struct sockaddr_storage *));
+extern char * socktohost        P((struct sockaddr_storage *));
 extern	int	octtoint	P((const char *, u_long *));
 extern	u_long	ranp2		P((int));
-extern	char *	refnumtoa	P((u_int32));
+extern	char *	refnumtoa	P((struct sockaddr_storage *));
 extern	int	tsftomsu	P((u_long, int));
 extern	char *	uinttoa		P((u_long));
 
-extern	int	decodenetnum	P((const char *, u_int32 *));
+extern	int	decodenetnum	P((const char *, struct sockaddr_storage *));
 
 extern	const char *	FindConfig	P((const char *));
 
@@ -142,7 +147,7 @@ extern HANDLE	hServDoneEvent;
 #endif
 
 /* systime.c */
-extern int	systime_10ms_ticks;	/* adj sysclock in 10ms increments */
+extern double	sys_tick;		/* adjtime() resolution */
 
 /* version.c */
 extern const char *Version;		/* version declaration */
