@@ -10,9 +10,6 @@
 #include "ntp_refclock.h"
 #include "recvbuff.h"
 
-#define MAXINTERFACES	512
-#define MAXFILENAME	128	/* maximum length of a file name */
-
 #ifdef SYS_WINNT
 #define exit service_exit
 extern	void	service_exit	(int);
@@ -21,8 +18,6 @@ void	service_main	(DWORD, LPTSTR *);
 void	service_ctrl	(DWORD);
 void	worker_thread	(void *);
 #define sleep(x) Sleep((DWORD) x * 1000 /* milliseconds */ );
-#else
-#define closesocket close
 #endif /* SYS_WINNT */
 
 /* ntp_config.c */
@@ -30,9 +25,9 @@ extern	void	getconfig	P((int, char **));
 
 /* ntp_config.c */
 extern	void	ctl_clr_stats	P((void));
-extern	int	ctlclrtrap	P((struct sockaddr_in *, struct interface *, int));
+extern	int	ctlclrtrap	P((struct sockaddr_storage *, struct interface *, int));
 extern	u_short ctlpeerstatus	P((struct peer *));
-extern	int	ctlsettrap	P((struct sockaddr_in *, struct interface *, int, int));
+extern	int	ctlsettrap	P((struct sockaddr_storage *, struct interface *, int, int));
 extern	u_short ctlsysstatus	P((void));
 extern	void	init_control	P((void));
 extern	void	process_control P((struct recvbuf *, int));
@@ -65,30 +60,30 @@ struct ctl_var {
 #define	WO	(CAN_WRITE)
 #define	RW	(CAN_READ|CAN_WRITE)
 
-extern  char *  add_var P((struct ctl_var **, unsigned long, int));
+extern  char *  add_var P((struct ctl_var **, u_long, u_short));
 extern  void    free_varlist P((struct ctl_var *));
-extern  void    set_var P((struct ctl_var **, const char *, unsigned long, int));
-extern  void    set_sys_var P((char *, unsigned long, int));
+extern  void    set_var P((struct ctl_var **, const char *, u_long, u_short));
+extern  void    set_sys_var P((char *, u_long, u_short));
 
 /* ntp_intres.c */
-extern	void	ntp_res_name	P((u_int32, u_short));
+extern	void	ntp_res_name	P((struct sockaddr_storage, u_short));
 extern	void	ntp_res_recv	P((void));
 extern	void	ntp_intres	P((void));
 
 /* ntp_io.c */
-extern	struct interface *findinterface P((struct sockaddr_in *));
-extern  struct interface *findbcastinter P((struct sockaddr_in *));
+extern	struct interface *findinterface P((struct sockaddr_storage *));
+extern  struct interface *findbcastinter P((struct sockaddr_storage *));
 
 extern	void	init_io 	P((void));
 extern	void	input_handler	P((l_fp *));
 extern	void	io_clr_stats	P((void));
 extern	void	io_setbclient	P((void));
 extern	void	io_unsetbclient P((void));
-extern	void	io_multicast_add P((u_int32));
-extern	void	io_multicast_del P((u_int32));
+extern	void	io_multicast_add P((struct sockaddr_storage));
+extern	void	io_multicast_del P((struct sockaddr_storage));
 extern	void	kill_asyncio	 P((void));
 
-extern	void	sendpkt 	P((struct sockaddr_in *, struct interface *, int, struct pkt *, int));
+extern	void	sendpkt 	P((struct sockaddr_storage *, struct interface *, int, struct pkt *, int));
 #ifdef HAVE_SIGNALED_IO
 extern	void	wait_for_signal P((void));
 extern	void	unblock_io_and_alarm P((void));
@@ -122,15 +117,15 @@ extern	void	ntp_monitor P((struct recvbuf *));
 
 /* ntp_peer.c */
 extern	void	init_peer	P((void));
-extern	struct peer *findexistingpeer P((struct sockaddr_in *, struct peer *, int));
-extern	struct peer *findpeer	P((struct sockaddr_in *, struct interface *, int, int, int *));
+extern	struct peer *findexistingpeer P((struct sockaddr_storage *, struct peer *, int));
+extern	struct peer *findpeer	P((struct sockaddr_storage *, struct interface *, int, int, int *));
 extern	struct peer *findpeerbyassoc P((u_int));
-extern	struct peer *newpeer	P((struct sockaddr_in *, struct interface *, int, int, int, int, u_int, u_int, int, keyid_t));
+extern	struct peer *newpeer	P((struct sockaddr_storage *, struct interface *, int, int, int, int, u_int, u_char, int, keyid_t));
 extern	void	peer_all_reset	P((void));
 extern	void	peer_clr_stats	P((void));
-extern	struct peer *peer_config P((struct sockaddr_in *, struct interface *, int, int, int, int, u_int, int, keyid_t, u_char *));
+extern	struct peer *peer_config P((struct sockaddr_storage *, struct interface *, int, int, int, int, u_int, int, keyid_t, u_char *));
 extern	void	peer_reset	P((struct peer *));
-extern	int 	peer_unconfig	P((struct sockaddr_in *, struct interface *, int));
+extern	int 	peer_unconfig	P((struct sockaddr_storage *, struct interface *, int));
 extern	void	unpeer		P((struct peer *));
 extern	void	clear_all	P((void));
 #ifdef OPENSSL
@@ -141,26 +136,27 @@ extern	void	resetmanycast	P((void));
 
 /* ntp_crypto.c */
 #ifdef OPENSSL
-extern	void	crypto_recv	P((struct peer *, struct recvbuf *, int));
-extern	int	crypto_xmit	P((struct pkt *, int, u_int32 *, keyid_t, u_int));
-extern	keyid_t	session_key	P((struct sockaddr_in *, struct sockaddr_in *, keyid_t, keyid_t, u_long));
+extern	int	crypto_recv	P((struct peer *, struct recvbuf *));
+extern	int	crypto_xmit	P((struct pkt *, struct sockaddr_storage *, int, struct exten *, keyid_t));
+extern	keyid_t	session_key	P((struct sockaddr_storage *, struct sockaddr_storage *, keyid_t, keyid_t, u_long));
 extern	void	make_keylist	P((struct peer *, struct interface *));
 extern	void	key_expire	P((struct peer *));
-extern	void	crypto_sign	P((void));
+extern	void	crypto_update	P((void));
 extern	void	crypto_config	P((int, char *));
 extern	void	crypto_setup	P((void));
+extern	u_int	crypto_ident	P((struct peer *));
+extern	struct exten *crypto_args P((struct peer *, u_int, u_char *));
 extern	int	crypto_public	P((struct peer *, u_char *, u_int));
-extern	struct cert_info *cert_parse P((u_char *, u_int));
-extern	void	cert_free	P((struct cert_info *));
 extern	void	value_free	P((struct value *));
 #endif /* OPENSSL */
 
 /* ntp_proto.c */
 extern	void	transmit	P((struct peer *));
 extern	void	receive 	P((struct recvbuf *));
-extern	void	peer_clear	P((struct peer *));
+extern	void	peer_clear	P((struct peer *, char *));
 extern	void 	process_packet	P((struct peer *, struct pkt *, l_fp *));
 extern	void	clock_select	P((void));
+extern	void	kod_proto	P((void));
 
 /*
  * there seems to be a bug in the IRIX 4 compiler which prevents
@@ -173,7 +169,7 @@ extern	void	poll_update P((struct peer *, int));
 extern	void	clear		P((struct peer *));
 extern	void	clock_filter	P((struct peer *, double, double, double));
 extern	void	init_proto	P((void));
-extern	void	proto_config	P((int, u_long, double));
+extern	void	proto_config	P((int, u_long, double, struct sockaddr_storage*));
 extern	void	proto_clr_stats P((void));
 
 #ifdef	REFCLOCK
@@ -191,8 +187,8 @@ extern	void	process_private P((struct recvbuf *, int));
 
 /* ntp_restrict.c */
 extern	void	init_restrict	P((void));
-extern	int 	restrictions	P((struct sockaddr_in *));
-extern	void	hack_restrict	P((int, struct sockaddr_in *, struct sockaddr_in *, int, int));
+extern	int 	restrictions	P((struct sockaddr_storage *));
+extern	void	hack_restrict	P((int, struct sockaddr_storage *, struct sockaddr_storage *, int, int));
 
 /* ntp_timer.c */
 extern	void	init_timer	P((void));
@@ -207,11 +203,13 @@ extern	l_fp	sys_revoketime;
 extern	void	init_util	P((void));
 extern	void	hourly_stats	P((void));
 extern	void	stats_config	P((int, char *));
-extern	void	record_peer_stats P((struct sockaddr_in *, int, double, double, double, double));
+extern	void	record_peer_stats P((struct sockaddr_storage *, int, double, double, double, double));
 extern	void	record_loop_stats P((double, double, double, double, int));
-extern	void	record_clock_stats P((struct sockaddr_in *, const char *));
-extern	void	record_raw_stats P((struct sockaddr_in *, struct sockaddr_in *, l_fp *, l_fp *, l_fp *, l_fp *));
-extern	void	record_crypto_stats P((struct sockaddr_in *, const char *));
+extern	void	record_clock_stats P((struct sockaddr_storage *, const char *));
+extern	void	record_raw_stats P((struct sockaddr_storage *, struct sockaddr_storage *, l_fp *, l_fp *, l_fp *, l_fp *));
+extern	void	record_sys_stats P((void));
+extern	void	record_crypto_stats P((struct sockaddr_storage *, const char *));
+extern  int	sock_hash P((struct sockaddr_storage *));
 
 /*
  * Variable declarations for ntpd.
@@ -271,7 +269,8 @@ extern u_long	io_timereset;		/* time counters were reset */
 /*
  * Interface stuff
  */
-extern struct interface *any_interface;	/* default interface */
+extern struct interface *any_interface;	/* default ipv4 interface */
+extern struct interface *any6_interface;/* default ipv6 interface */
 extern struct interface *loopback_interface; /* loopback interface */
 
 /*
@@ -301,21 +300,20 @@ extern int	kern_enable;		/* kernel support enabled */
 extern int	pps_enable;		/* kernel PPS discipline enabled */
 extern int	ext_enable;		/* external clock enabled */
 extern int	cal_enable;		/* refclock calibrate enable */
-extern int	allow_step;		/* allow step correction */
 extern int	allow_panic;		/* allow panic correction */
 extern int	mode_ntpdate;		/* exit on first clock set */
 extern int	peer_ntpdate;		/* count of ntpdate peers */
+extern int	forground_process;	/* run the process in the forground */
 
 /*
  * Clock state machine variables
  */
 extern u_char	sys_poll;		/* system poll interval (log2 s) */
-extern u_char	sys_minpoll;		/* min system poll interval (log2 s) */
 extern int	state;			/* clock discipline state */
 extern int	tc_counter;		/* poll-adjust counter */
 extern u_long	last_time;		/* time of last clock update (s) */
 extern double	last_offset;		/* last clock offset (s) */
-extern u_char	allan_xpt;		/* Allan intercept (s) */
+extern double	allan_xpt;		/* Allan intercept (s) */
 extern double	sys_error;		/* system RMS error (s) */
 extern double	sys_jitter;		/* system RMS jitter (s) */
 
@@ -366,20 +364,27 @@ extern double	sys_bdelay; 		/* broadcast client default delay */
 extern int	sys_authenticate;	/* requre authentication for config */
 extern l_fp	sys_authdelay;		/* authentication delay */
 extern keyid_t	sys_private;		/* private value for session seed */
-extern int	sys_manycastserver;	/* 1 => respond to manycast client pkts */
+extern int	sys_manycastserver;	/* respond to manycast client pkts */
+extern int	sys_minclock;		/* minimum survivors */
+extern int	sys_minsane;		/* minimum candidates */
+extern int	sys_floor;		/* cluster stratum floor */
+extern int	sys_ceiling;		/* cluster stratum ceiling */
+extern u_char	sys_ttl[];		/* ttl mapping vector */
+extern int	sys_ttlmax;		/* max ttl mapping vector index */
 
 /*
  * Statistics counters
  */
 extern u_long	sys_stattime;		/* time when we started recording */
-extern u_long	sys_badstratum; 	/* packets with invalid stratum */
-extern u_long	sys_oldversionpkt;	/* old version packets received */
-extern u_long	sys_newversionpkt;	/* new version packets received */
+extern u_long	sys_restricted;	 	/* restricted packets */
+extern u_long	sys_oldversionpkt;	/* old version packets */
+extern u_long	sys_newversionpkt;	/* new version packets  */
 extern u_long	sys_unknownversion;	/* don't know version packets */
-extern u_long	sys_badlength;		/* packets with bad length */
+extern u_long	sys_badlength;		/* bad length or format */
 extern u_long	sys_processed;		/* packets processed */
-extern u_long	sys_badauth;		/* packets dropped because of auth */
-extern u_long	sys_limitrejected;	/* pkts rejected due to client count per net */
+extern u_long	sys_badauth;		/* bad authentication */
+extern u_long	sys_limitrejected;	/* rate limit exceeded */
+extern u_long	sys_received;		/* packets received */
 
 /* ntp_refclock.c */
 #ifdef REFCLOCK
@@ -392,9 +397,10 @@ extern int	fdpps;			/* pps file descriptor */
 extern keyid_t	info_auth_keyid;	/* keyid used to authenticate requests */
 
 /* ntp_restrict.c */
-extern struct restrictlist *restrictlist; /* the restriction list */
-extern u_long	client_limit;
-extern u_long	client_limit_period;
+extern struct restrictlist *restrictlist; /* the ipv4 restriction list */
+extern struct restrictlist6 *restrictlist6; /* the ipv6 restriction list */
+extern u_long	res_min_interval;
+extern u_long	res_avg_interval;
 
 /* ntp_timer.c */
 extern volatile int alarm_flag;		/* alarm flag */
@@ -412,6 +418,11 @@ extern int	stats_control;		/* write stats to fileset? */
 extern volatile int debug;		/* debugging flag */
 extern int	nofork;			/* no-fork flag */
 extern int 	initializing;		/* initializing flag */
+#ifdef HAVE_CLOCKCTL
+extern char *user;			/* user to switch to */
+extern char *group;			/* group to switch to */
+extern char *chrootdir;			/* directory to chroot to */
+#endif
 
 /* refclock_conf.c */
 #ifdef REFCLOCK
