@@ -18,10 +18,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#ifdef OPENSSL
-#include "ntp_crypto.h"
-#endif /* OPENSSL */
-
 /*
  * Structure to hold request procedure information
  */
@@ -56,7 +52,7 @@ static	void	ctl_putuint	P((const char *, u_long));
 static	void	ctl_puthex	P((const char *, u_long));
 static	void	ctl_putint	P((const char *, long));
 static	void	ctl_putts	P((const char *, l_fp *));
-static	void	ctl_putadr	P((const char *, u_int32));
+static	void	ctl_putadr	P((const char *, u_int32, struct sockaddr_storage*));
 static	void	ctl_putid	P((const char *, char *));
 static	void	ctl_putarray	P((const char *, double *, int));
 static	void	ctl_putsys	P((int));
@@ -74,7 +70,7 @@ static	void	read_clock_status P((struct recvbuf *, int));
 static	void	write_clock_status P((struct recvbuf *, int));
 static	void	set_trap	P((struct recvbuf *, int));
 static	void	unset_trap	P((struct recvbuf *, int));
-static	struct ctl_trap *ctlfindtrap P((struct sockaddr_in *,
+static	struct ctl_trap *ctlfindtrap P((struct sockaddr_storage *,
 				    struct interface *));
 
 static	struct ctl_proc control_codes[] = {
@@ -118,7 +114,7 @@ static struct ctl_var sys_var[] = {
 	{ CS_FLAGS,	RO, "flags" },		/* 20 */
 	{ CS_HOST,	RO, "hostname" },	/* 21 */
 	{ CS_PUBLIC,	RO, "hostkey" },	/* 22 */
-	{ CS_CERTIF,	RO, "certificate" },	/* 23 */
+	{ CS_CERTIF,	RO, "cert" },		/* 23 */
 	{ CS_REVTIME,	RO, "refresh" },	/* 24 */
 	{ CS_LEAPTAB,	RO, "leapseconds" },	/* 25 */
 	{ CS_TAI,	RO, "tai" },		/* 26 */
@@ -156,10 +152,10 @@ static	u_char def_sys_var[] = {
 	CS_HOST,
 	CS_DIGEST,
 	CS_FLAGS,
-	CS_CERTIF,
 	CS_PUBLIC,
 	CS_REVTIME,
 	CS_LEAPTAB,
+	CS_CERTIF,
 #endif /* OPENSSL */
 	0
 };
@@ -206,19 +202,17 @@ static struct ctl_var peer_var[] = {
 	{ CP_FILTERROR,	RO, "filtdisp=" },	/* 34 */
 	{ CP_FLASH,	RO, "flash" },		/* 35 */
 	{ CP_TTL,	RO, "ttl" },		/* 36 */
-	{ CP_TTLMAX,	RO, "ttlmax" },		/* 37 */
-	{ CP_VARLIST,	RO, "peer_var_list" },	/* 38 */
+	{ CP_VARLIST,	RO, "peer_var_list" },	/* 37 */
 #ifdef OPENSSL
 	{ CP_FLAGS,	RO, "flags" },		/* 38 */
 	{ CP_HOST,	RO, "hostname" },	/* 39 */
-	{ CP_CERTIF,	RO, "certificate" },	/* 40 */
-	{ CP_SESKEY,	RO, "cookie" },		/* 41 */
-	{ CP_INITSEQ,	RO, "initsequence" },   /* 42 */
-	{ CP_INITKEY,	RO, "initkey" },	/* 43 */
-	{ CP_INITTSP,	RO, "timestamp" },	/* 44 */
-	{ CP_DIGEST,	RO, "signature" },	/* 45 */
+	{ CP_INITSEQ,	RO, "initsequence" },   /* 40 */
+	{ CP_INITKEY,	RO, "initkey" },	/* 41 */
+	{ CP_INITTSP,	RO, "timestamp" },	/* 42 */
+	{ CP_DIGEST,	RO, "signature" },	/* 43 */
+	{ CP_IDENT,	RO, "identity" },	/* 44 */
 #endif /* OPENSSL */
-	{ 0,		EOV, ""  }		/* 46 */
+	{ 0,		EOV, "" }		/* 38/43 */
 };
 
 
@@ -245,7 +239,6 @@ static u_char def_peer_var[] = {
 	CP_FLASH,
 	CP_KEYID,
 	CP_TTL,
-	CP_TTLMAX,
 	CP_OFFSET,
 	CP_DELAY,
 	CP_DISPERSION,
@@ -261,8 +254,7 @@ static u_char def_peer_var[] = {
 	CP_HOST,
 	CP_DIGEST,
 	CP_FLAGS,
-	CP_CERTIF,
-	CP_SESKEY,
+	CP_IDENT,
 	CP_INITSEQ,
 #endif /* OPENSSL */
 	0
@@ -397,6 +389,10 @@ static u_char clocktypes[] = {
 	CTL_SST_TS_UHF, 	/* REFCLK_HOPF_SERIAL (38) */
 	CTL_SST_TS_UHF,		/* REFCLK_HOPF_PCI (39) */
 	CTL_SST_TS_LF,		/* REFCLK_JJY (40) */
+	CTL_SST_TS_UHF,		/* REFCLK_TT560 (41) */
+	CTL_SST_TS_UHF,		/* REFCLK_ZYFER (42) */
+	CTL_SST_TS_UHF,		/* REFCLK_RIPENCC (43) */
+	CTL_SST_TS_UHF,		/* REFCLK_NEOCLOCK4X (44) */
 };
 
 
@@ -447,7 +443,7 @@ static u_char * datapt;
 static u_char * dataend;
 static int	datalinelen;
 static int	datanotbinflag;
-static struct sockaddr_in *rmt_addr;
+static struct sockaddr_storage *rmt_addr;
 static struct interface *lcl_inter;
 
 static u_char	res_authenticate;
@@ -737,8 +733,8 @@ ctlclkstatus(
 	struct refclockstat *this_clock
 	)
 {
-	return ((u_short)(this_clock->currentstatus) << 8) |
-	    (u_short)(this_clock->lastevent);
+	return ((u_short)(((this_clock->currentstatus) << 8) |
+	    (this_clock->lastevent)));
 }
 
 
@@ -1071,12 +1067,13 @@ ctl_putts(
 
 
 /*
- * ctl_putadr - write a dotted quad IP address into the response
+ * ctl_putadr - write an IP address into the response
  */
 static void
 ctl_putadr(
 	const char *tag,
-	u_int32 addr
+	u_int32 addr32,
+	struct sockaddr_storage* addr
 	)
 {
 	register char *cp;
@@ -1089,7 +1086,10 @@ ctl_putadr(
 		*cp++ = *cq++;
 
 	*cp++ = '=';
-	cq = numtoa(addr);
+	if (addr == NULL)
+		cq = numtoa(addr32);
+	else
+		cq = stoa(addr);
 	while (*cq != '\0')
 		*cp++ = *cq++;
 	ctl_putdata(buffer, (unsigned)( cp - buffer ), 0);
@@ -1136,7 +1136,6 @@ ctl_putarray(
 	register const char *cq;
 	char buffer[200];
 	int i;
-
 	cp = buffer;
 	cq = tag;
 	while (*cq != '\0')
@@ -1164,6 +1163,10 @@ ctl_putsys(
 {
 	l_fp tmp;
 	char str[256];
+#ifdef OPENSSL
+	struct cert_info *cp;
+	char cbuf[256];
+#endif /* OPENSSL */
 
 	switch (varid) {
 
@@ -1190,8 +1193,8 @@ ctl_putsys(
 		break;
 
 	case CS_REFID:
-		if (sys_stratum > 1)
-			ctl_putadr(sys_var[CS_REFID].text, sys_refid);
+		if (sys_stratum > 1 && sys_stratum < STRATUM_UNSPEC)
+			ctl_putadr(sys_var[CS_REFID].text, sys_refid, NULL);
 		else
 			ctl_putid(sys_var[CS_REFID].text,
 			    (char *)&sys_refid);
@@ -1352,19 +1355,25 @@ ctl_putsys(
 		break;
 
 	case CS_CERTIF:
-		if (cinfo.fstamp != 0)
-			ctl_putuint(sys_var[CS_CERTIF].text, cinfo.fstamp);
+		for (cp = cinfo; cp != NULL; cp = cp->link) {
+			sprintf(cbuf, "%s %s 0x%x %u", cp->subject,
+			    cp->issuer, cp->flags,
+			    ntohl(cp->cert.fstamp));
+			ctl_putstr(sys_var[CS_CERTIF].text, cbuf,
+			    strlen(cbuf));
+		}
 		break;
 
 	case CS_PUBLIC:
-		if (host.fstamp != 0)
-			ctl_putuint(sys_var[CS_PUBLIC].text, host.fstamp);
+		if (hostval.fstamp != 0)
+			ctl_putuint(sys_var[CS_PUBLIC].text,
+			    ntohl(hostval.fstamp));
 		break;
 
 	case CS_REVTIME:
-		if (host.tstamp != 0)
+		if (hostval.tstamp != 0)
 			ctl_putuint(sys_var[CS_REVTIME].text,
-			    ntohl(host.tstamp));
+			    ntohl(hostval.tstamp));
 		break;
 
 	case CS_LEAPTAB:
@@ -1411,24 +1420,24 @@ ctl_putpeer(
 		break;
 
 	case CP_SRCADR:
-		ctl_putadr(peer_var[CP_SRCADR].text,
-		    peer->srcadr.sin_addr.s_addr);
+		ctl_putadr(peer_var[CP_SRCADR].text, 0,
+		    &peer->srcadr);
 		break;
 
 	case CP_SRCPORT:
 		ctl_putuint(peer_var[CP_SRCPORT].text,
-		    ntohs(peer->srcadr.sin_port));
+		    ntohs(((struct sockaddr_in*)&peer->srcadr)->sin_port));
 		break;
 
 	case CP_DSTADR:
-		ctl_putadr(peer_var[CP_DSTADR].text,
-		    peer->dstadr->sin.sin_addr.s_addr);
+		ctl_putadr(peer_var[CP_DSTADR].text, 0,
+		    &(peer->dstadr->sin));
 		break;
 
 	case CP_DSTPORT:
 		ctl_putuint(peer_var[CP_DSTPORT].text,
 		    (u_long)(peer->dstadr ?
-		    ntohs(peer->dstadr->sin.sin_port) : 0));
+		    ntohs(((struct sockaddr_in*)&peer->dstadr->sin)->sin_port) : 0));
 		break;
 
 	case CP_LEAP:
@@ -1468,19 +1477,21 @@ ctl_putpeer(
 
 	case CP_REFID:
 		if (peer->flags & FLAG_REFCLOCK) {
-			if (peer->stratum > 0)
-			    ctl_putadr(peer_var[CP_REFID].text,
-			        peer->refid);
+			if (peer->stratum > 0 && peer->stratum <
+			    STRATUM_UNSPEC)
+				ctl_putadr(peer_var[CP_REFID].text,
+				    peer->refid, NULL);
 			else
-			    ctl_putid(peer_var[CP_REFID].text,
-			        (char *)&peer->refid);
+				ctl_putid(peer_var[CP_REFID].text,
+				   (char *)&peer->refid);
 		} else {
-			if (peer->stratum > 1)
-			    ctl_putadr(peer_var[CP_REFID].text,
-				peer->refid);
+			if (peer->stratum > 1 && peer->stratum <
+			    STRATUM_UNSPEC)
+				ctl_putadr(peer_var[CP_REFID].text,
+				    peer->refid, NULL);
 			else
-			    ctl_putid(peer_var[CP_REFID].text,
-				(char *)&peer->refid);
+				ctl_putid(peer_var[CP_REFID].text,
+				    (char *)&peer->refid);
 		}
 		break;
 
@@ -1509,15 +1520,7 @@ ctl_putpeer(
 		break;
 
 	case CP_TTL:
-		if (!(peer->cast_flags & MDF_ACAST))
-			break;
-		ctl_putint(peer_var[CP_TTL].text, peer->ttl);
-		break;
-
-	case CP_TTLMAX:
-		if (!(peer->cast_flags & (MDF_MCAST | MDF_ACAST)))
-			break;
-		ctl_putint(peer_var[CP_TTLMAX].text, peer->ttlmax);
+		ctl_putint(peer_var[CP_TTL].text, sys_ttl[peer->ttl]);
 		break;
 
 	case CP_VALID:
@@ -1635,21 +1638,15 @@ ctl_putpeer(
 		break;
 
 	case CP_HOST:
-		if (peer->keystr != NULL)
-			ctl_putstr(peer_var[CP_HOST].text, peer->keystr,
-			    strlen(peer->keystr));
+		if (peer->subject != NULL)
+			ctl_putstr(peer_var[CP_HOST].text, peer->subject,
+			    strlen(peer->subject));
 		break;
 
-	case CP_CERTIF:
-		if (peer->cinfo.fstamp != 0)
-			ctl_putuint(peer_var[CP_CERTIF].text,
-			    peer->cinfo.fstamp);
-		break;
-
-	case CP_SESKEY:
-		if (peer->pcookie != 0)
-			ctl_puthex(peer_var[CP_SESKEY].text,
-			    peer->pcookie);
+	case CP_IDENT:
+		if (peer->issuer != NULL)
+			ctl_putstr(peer_var[CP_IDENT].text, peer->issuer,
+			    strlen(peer->issuer));
 		break;
 
 	case CP_INITSEQ:
@@ -1658,7 +1655,7 @@ ctl_putpeer(
 		ctl_putint(peer_var[CP_INITSEQ].text, ap->seq);
 		ctl_puthex(peer_var[CP_INITKEY].text, ap->key);
 		ctl_putuint(peer_var[CP_INITTSP].text,
-		    peer->recval.tstamp);
+		    ntohl(peer->recval.tstamp));
 		break;
 #endif /* OPENSSL */
 	}
@@ -1730,7 +1727,7 @@ ctl_putclock(
 		if (mustput || (clock_stat->haveflags & CLK_HAVEVAL2)) {
 			if (clock_stat->fudgeval1 > 1)
 				ctl_putadr(clock_var[CC_FUDGEVAL2].text,
-				    (u_int32)clock_stat->fudgeval2);
+				    (u_int32)clock_stat->fudgeval2, NULL);
 			else
 				ctl_putid(clock_var[CC_FUDGEVAL2].text,
 				    (char *)&clock_stat->fudgeval2);
@@ -1889,7 +1886,7 @@ ctl_getitem(
 							numctlbadpkts++;
 							msyslog(LOG_WARNING,
 		"Possible 'ntpdx' exploit from %s:%d (possibly spoofed)\n",
-		inet_ntoa(rmt_addr->sin_addr), ntohs(rmt_addr->sin_port)
+		stoa(rmt_addr), SRCPORT(rmt_addr)
 								);
 							return (0);
 						}
@@ -2143,7 +2140,7 @@ write_variables(
 	register struct ctl_var *v;
 	register int ext_var;
 	char *valuep;
-	long val;
+	long val = 0;
 
 	/*
 	 * If he's trying to write into a peer tell him no way
@@ -2447,7 +2444,7 @@ unset_trap(
  */
 int
 ctlsettrap(
-	struct sockaddr_in *raddr,
+	struct sockaddr_storage *raddr,
 	struct interface *linter,
 	int traptype,
 	int version
@@ -2552,7 +2549,7 @@ ctlsettrap(
 	tptouse->tr_sequence = 1;
 	tptouse->tr_addr = *raddr;
 	tptouse->tr_localaddr = linter;
-	tptouse->tr_version = version;
+	tptouse->tr_version = (u_char) version;
 	tptouse->tr_flags = TRAP_INUSE;
 	if (traptype == TRAP_TYPE_CONFIG)
 		tptouse->tr_flags |= TRAP_CONFIGURED;
@@ -2568,7 +2565,7 @@ ctlsettrap(
  */
 int
 ctlclrtrap(
-	struct sockaddr_in *raddr,
+	struct sockaddr_storage *raddr,
 	struct interface *linter,
 	int traptype
 	)
@@ -2593,18 +2590,18 @@ ctlclrtrap(
  */
 static struct ctl_trap *
 ctlfindtrap(
-	struct sockaddr_in *raddr,
+	struct sockaddr_storage *raddr,
 	struct interface *linter
 	)
 {
 	register struct ctl_trap *tp;
 
 	for (tp = ctl_trap; tp < &ctl_trap[CTL_MAXTRAPS]; tp++) {
-		if (tp->tr_flags & TRAP_INUSE && NSRCADR(raddr) ==
-		    NSRCADR(&tp->tr_addr) && NSRCPORT(raddr) ==
-		    NSRCPORT(&tp->tr_addr) && linter ==
-		    tp->tr_localaddr)
-			return (tp);
+		if ((tp->tr_flags & TRAP_INUSE)
+		    && (NSRCPORT(raddr) == NSRCPORT(&tp->tr_addr))
+		    && SOCKCMP(raddr, &tp->tr_addr)
+	 	    && (linter == tp->tr_localaddr) )
+		return (tp);
 	}
 	return (struct ctl_trap *)NULL;
 }
@@ -2647,10 +2644,10 @@ report_event(
 
 #ifdef REFCLOCK
 		if (ISREFCLOCKADR(&peer->srcadr))
-			src = refnumtoa(peer->srcadr.sin_addr.s_addr);
+			src = refnumtoa(&peer->srcadr);
 		else
 #endif
-			src = ntoa(&peer->srcadr);
+			src = stoa(&peer->srcadr);
 
 		peer->last_event = (u_char)(err & ~PEER_EVENT);
 		if (peer->num_events < CTL_PEER_MAXEVENTS)
@@ -2740,12 +2737,13 @@ report_event(
 		/*
 		 * Dump it all. Later, maybe less.
 		 */
-		for (i = 1; i <= CP_MAXCODE; i++)
+		for (i = 1; i <= CP_MAXCODE; i++) {
 #ifdef OPENSSL
 			if (i > CP_VARLIST)
 				continue;
 #endif /* OPENSSL */
 			ctl_putpeer(i, peer);
+		}
 #ifdef REFCLOCK
 		/*
 		 * for clock exception events: add clock variables to
@@ -2825,7 +2823,7 @@ char *
 add_var(
 	struct ctl_var **kv,
 	u_long size,
-	int def
+	u_short def
 	)
 {
 	register u_long c;
@@ -2854,7 +2852,7 @@ set_var(
 	struct ctl_var **kv,
 	const char *data,
 	u_long size,
-	int def
+	u_short def
 	)
 {
 	register struct ctl_var *k;
@@ -2865,7 +2863,8 @@ set_var(
 	if (!data || !size)
 		return;
 
-	if ((k = *kv)) {
+	k = *kv;
+	if (k != NULL) {
 		while (!(k->flags & EOV)) {
 			s = data;
 			t = k->text;
@@ -2900,7 +2899,7 @@ void
 set_sys_var(
 	char *data,
 	u_long size,
-	int def
+	u_short def
 	)
 {
 	set_var(&ext_sys_var, data, size, def);
