@@ -11,6 +11,10 @@
 #include "ntp_io.h"
 #include "ntp_stdlib.h"
 
+#ifdef SIM
+#include "ntpsim.h"
+#endif
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -40,6 +44,7 @@
 # include <process.h>
 # include <io.h>
 # include "../libntp/log.h"
+# include <clockstuff.h>
 # include <crtdbg.h>
 #endif /* SYS_WINNT */
 #if defined(HAVE_RTPRIO)
@@ -99,9 +104,11 @@
 # include <sys/ci/ciioctl.h>
 #endif
 
-#ifdef OPENSSL
-#include "ntp_crypto.h"
-#endif /* OPENSSL */
+#ifdef HAVE_CLOCKCTL
+# include <ctype.h>
+# include <grp.h>
+# include <pwd.h>
+#endif
 
 /*
  * Signals we catch for debugging.	If not debugging we ignore them.
@@ -145,11 +152,27 @@ int priority_done = 2;		/* 0 - Set priority */
  */
 volatile int debug;
 
+/*
+ * Set the processing not to be in the forground
+ */
+int forground_process = FALSE;
+
 /*
  * No-fork flag.  If set, we do not become a background daemon.
  */
 int nofork;
 
+#ifdef HAVE_CLOCKCTL
+char *user = NULL;		/* User to switch to */
+char *group = NULL;		/* group to switch to */
+char *chrootdir = NULL;		/* directory to chroot to */
+int sw_uid;
+int sw_gid;
+char *endp;  
+struct group *gr;
+struct passwd *pw; 
+#endif /* HAVE_CLOCKCTL */
+
 /*
  * Initializing flag.  All async routines watch this and only do their
  * thing when it is clear.
@@ -176,8 +199,10 @@ static	RETSIGTYPE	finish		P((int));
 #endif	/* SIGDIE2 */
 
 #ifdef	DEBUG
+#ifndef SYS_WINNT
 static	RETSIGTYPE	moredebug	P((int));
 static	RETSIGTYPE	lessdebug	P((int));
+#endif
 #else /* not DEBUG */
 static	RETSIGTYPE	no_debug	P((int));
 #endif	/* not DEBUG */
@@ -185,7 +210,16 @@ static	RETSIGTYPE	no_debug	P((int));
 int 		ntpdmain		P((int, char **));
 static void	set_process_priority	P((void));
 
-
+#ifdef SIM
+int
+main(
+	int argc,
+	char *argv[]
+	)
+{
+	return ntpsim(argc, argv);
+}
+#else /* SIM */
 #ifdef NO_MAIN_ALLOWED
 CALL(ntpd,"ntpd",ntpdmain);
 #else
@@ -198,6 +232,7 @@ main(
 	return ntpdmain(argc, argv);
 }
 #endif
+#endif /* SIM */
 
 #ifdef _AIX
 /*
@@ -519,8 +554,9 @@ service_main(
 	if(!debug)
 	{
 		/* register our service control handler */
-		if (!(sshStatusHandle = RegisterServiceCtrlHandler( TEXT("NetworkTimeProtocol"),
-									(LPHANDLER_FUNCTION)service_ctrl)))
+		sshStatusHandle = RegisterServiceCtrlHandler( TEXT("NetworkTimeProtocol"),
+							(LPHANDLER_FUNCTION)service_ctrl);
+		if(sshStatusHandle == 0)
 		{
 			msyslog(LOG_ERR, "RegisterServiceCtrlHandler failed: %m");
 			return;
@@ -559,7 +595,7 @@ service_main(
 	debug = 0; /* will be immediately re-initialized 8-( */
 	getstartup(argc, argv); /* startup configuration, catch logfile this time */
 
-#if !defined(SYS_WINNT) && !defined(VMS)
+#if !defined(VMS)
 
 # ifndef LOG_DAEMON
 	openlog(cp, LOG_PID);
@@ -760,6 +796,64 @@ service_main(
 # endif  
 #endif
 
+#ifdef HAVE_CLOCKCTL
+	/* 
+	 * Drop super-user privileges and chroot now if the OS supports
+	 * non root clock control (only NetBSD for now).
+	 */
+	if (user != NULL) {
+	        if (isdigit((unsigned char)*user)) {
+	                sw_uid = (uid_t)strtoul(user, &endp, 0);
+	                if (*endp != '\0') 
+	                        goto getuser;
+	        } else {
+getuser:	
+	                if ((pw = getpwnam(user)) != NULL) {
+	                        sw_uid = pw->pw_uid;
+	                } else {
+	                        errno = 0;
+	                        msyslog(LOG_ERR, "Cannot find user `%s'", user);
+									exit (-1);
+	                }
+	        }
+	}
+	if (group != NULL) {
+	        if (isdigit((unsigned char)*group)) {
+	                sw_gid = (gid_t)strtoul(group, &endp, 0);
+	                if (*endp != '\0') 
+	                        goto getgroup;
+	        } else {
+getgroup:	
+	                if ((gr = getgrnam(group)) != NULL) {
+	                        sw_gid = pw->pw_gid;
+	                } else {
+	                        errno = 0;
+	                        msyslog(LOG_ERR, "Cannot find group `%s'", group);
+									exit (-1);
+	                }
+	        }
+	}
+	if (chrootdir && chroot(chrootdir)) {
+		msyslog(LOG_ERR, "Cannot chroot to `%s': %m", chrootdir);
+		exit (-1);
+	}
+	if (group && setgid(sw_gid)) {
+		msyslog(LOG_ERR, "Cannot setgid() to group `%s': %m", group);
+		exit (-1);
+	}
+	if (group && setegid(sw_gid)) {
+		msyslog(LOG_ERR, "Cannot setegid() to group `%s': %m", group);
+		exit (-1);
+	}
+	if (user && setuid(sw_uid)) {
+		msyslog(LOG_ERR, "Cannot setuid() to user `%s': %m", user);
+		exit (-1);
+	}
+	if (user && seteuid(sw_uid)) {
+		msyslog(LOG_ERR, "Cannot seteuid() to user `%s': %m", user);
+		exit (-1);
+	}
+#endif
 	/*
 	 * Report that we're up to any trappers
 	 */
@@ -783,7 +877,7 @@ service_main(
 #if defined(HAVE_IO_COMPLETION_PORT)
 		WaitHandles[0] = CreateEvent(NULL, FALSE, FALSE, NULL); /* exit reques */
 		WaitHandles[1] = get_timer_handle();
-	    WaitHandles[2] = get_io_event();
+		WaitHandles[2] = get_io_event();
 
 		for (;;) {
 			DWORD Index = WaitForMultipleObjectsEx(sizeof(WaitHandles)/sizeof(WaitHandles[0]), WaitHandles, FALSE, 1000, MWMO_ALERTABLE);
@@ -937,7 +1031,9 @@ service_main(
 		 */
 	}
 	exit(1); /* unreachable */
+#ifndef SYS_WINNT
 	return 1;		/* DEC OSF cc braindamage */
+#endif
 }
 
 
@@ -970,6 +1066,7 @@ finish(
 
 
 #ifdef DEBUG
+#ifndef SYS_WINNT
 /*
  * moredebug - increase debugging verbosity
  */
@@ -1005,6 +1102,7 @@ lessdebug(
 	}
 	errno = saved_errno;
 }
+#endif
 #else /* not DEBUG */
 /*
  * no_debug - We don't do the debug here.
