@@ -320,7 +320,7 @@ struct chuunit {
 	int	fd_audio;	/* audio port file descriptor */
 	double	comp[SIZE];	/* decompanding table */
 	int	port;		/* codec port */
-	int	gain;		/* codec input gain */
+	int	gain;		/* codec gain */
 	int	mongain;	/* codec monitor gain */
 	int	clipcnt;	/* sample clip count */
 	int	seccnt;		/* second interval counter */
@@ -419,7 +419,7 @@ chu_start(
 	/*
 	 * Open audio device.
 	 */
-	fd_audio = audio_init(DEVICE_AUDIO, AUDIO_BUFSIZ);
+	fd_audio = audio_init(DEVICE_AUDIO, AUDIO_BUFSIZ, unit);
 #ifdef DEBUG
 	if (fd_audio > 0 && debug)
 		audio_show();
@@ -501,8 +501,8 @@ chu_start(
 	if (debug > 1)
 		temp = P_TRACE;
 #endif
-	if (peer->ttlmax > 0) {
-		if (peer->ttlmax & 0x80)
+	if (peer->ttl > 0) {
+		if (peer->ttl & 0x80)
 			up->fd_icom = icom_init("/dev/icom", B1200,
 			    temp);
 		else
@@ -510,7 +510,7 @@ chu_start(
 			    temp);
 	}
 	if (up->fd_icom > 0) {
-		if (icom_freq(up->fd_icom, peer->ttlmax & 0x7f,
+		if (icom_freq(up->fd_icom, peer->ttl & 0x7f,
 		    qsy[up->chan]) < 0) {
 			NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
 			    msyslog(LOG_ERR,
@@ -1253,7 +1253,7 @@ chu_poll(
 	} else if (up->fd_icom > 0) {
 		up->dwell = 0;
 		up->chan = (up->chan + 1) % NCHAN;
-		icom_freq(up->fd_icom, peer->ttlmax & 0x7f,
+		icom_freq(up->fd_icom, peer->ttl & 0x7f,
 		    qsy[up->chan]);
 		sprintf(up->ident, "%.3f", qsy[up->chan]); 
 		sprintf(tbuf, "chu: QSY to %s MHz", up->ident);
@@ -1285,6 +1285,9 @@ chu_poll(
 	} else if (up->leap & 0x2) {
 		pp->leap = LEAP_ADDSECOND;
 		leapchar = 'L';
+	} else if (up->leap & 0x4) {
+		pp->leap = LEAP_DELSECOND;
+		leapchar = 'l';
 	} else {
 		pp->leap = LEAP_NOWARNING;
 	}
@@ -1317,8 +1320,8 @@ chu_poll(
 	 * correct and can be selected to discipline the clock.
 	 */
 	if (temp > 0) {
-		record_clock_stats(&peer->srcadr, pp->a_lastcode);
 		refclock_receive(peer);
+		record_clock_stats(&peer->srcadr, pp->a_lastcode);
 	} else if (pp->sloppyclockflag & CLK_FLAG4) {
 		record_clock_stats(&peer->srcadr, pp->a_lastcode);
 	}
@@ -1424,15 +1427,16 @@ chu_major(
 		up->errflg = CEVNT_BADREPLY;
 		return (0);
 	}
-	L_CLR(&offset);
 	if (!clocktime(pp->day, pp->hour, pp->minute, 0, GMT,
 	    up->tstamp[0].l_ui, &pp->yearstart, &offset.l_ui)) {
 		up->errflg = CEVNT_BADTIME;
 		return (0);
 	}
+	offset.l_uf = 0;
 	for (i = 0; i < up->ntstamp; i++)
-		refclock_process_offset(pp, offset, up->tstamp[i], FUDGE +
-		    pp->fudgetime1);
+		refclock_process_offset(pp, offset, up->tstamp[i],
+		    FUDGE + pp->fudgetime1);
+	pp->lastref = up->timestamp;
 	return (i);
 }
 
