@@ -892,7 +892,7 @@ mx4200_receive(
 		mx4200_debug(peer, "%4d-%03d %02d:%02d:%02d at %s, %.6f\n",
 		    pp->year, pp->day, pp->hour, pp->minute, pp->second,
 		    prettydate(&pp->lastrec), pp->offset);
-
+		pp->lastref = pp->lastrec;
 		refclock_receive(peer);
 
 		/*
@@ -1103,8 +1103,6 @@ mx4200_parse_t(
 	pp->hour   = hour;
 	pp->minute = minute;
 	pp->second = second;
-	pp->msec   = 0;
-	pp->usec   = 0;
 
 	/*
 	 * Toss if sentence is marked invalid
