@@ -28,16 +28,16 @@
  *					OPTIONS LIST	IB
  *
  *	      (Basic)				   (M12)
- *   COPYRIGHT 1991-1996 MOTOROLA INC.	COPYRIGHT 1991-2000 MOTOROLA INC.
- *   SFTW P/N # 98-P36830P		SFTW P/N # 61-G10002A
- *   SOFTWARE VER # 8			SOFTWARE VER # 1
- *   SOFTWARE REV # 8			SOFTWARE REV # 3
- *   SOFTWARE DATE  06 Aug 1996 	SOFTWARE DATE  Mar 13 2000
- *   MODEL #	B4121P1155		MODEL #    P143T12NR1
+ *   COPYRIGHT 1991-1994 MOTOROLA INC.	COPYRIGHT 1991-2000 MOTOROLA INC.
+ *   SFTW P/N # 98-P39949M		SFTW P/N # 61-G10002A
+ *   SOFTWARE VER # 5			SOFTWARE VER # 1
+ *   SOFTWARE REV # 0			SOFTWARE REV # 3
+ *   SOFTWARE DATE  20 JAN 1994 	SOFTWARE DATE  Mar 13 2000
+ *   MODEL #	A11121P116		MODEL #    P143T12NR1
  *   HDWR P/N # _			HWDR P/N # 1
- *   SERIAL #	SSG0226478		SERIAL #   P003UD
- *   MANUFACTUR DATE 7E02		MANUFACTUR DATE 0C27
- *   OPTIONS LIST    IB
+ *   SERIAL #	SSG0049809		SERIAL #   P003UD
+ *   MANUFACTUR DATE 417AMA199		MANUFACTUR DATE 0C27
+ *   OPTIONS LIST    AB
  *
  * --------------------------------------------------------------------------
  * This code uses the two devices
@@ -154,6 +154,14 @@ enum site_survey_state {
 	ONCORE_SS_DONE
 };
 
+enum antenna_state {
+      ONCORE_ANTENNA_UNKNOWN = -1,
+      ONCORE_ANTENNA_OK      =	0,
+      ONCORE_ANTENNA_OC      =	1,
+      ONCORE_ANTENNA_UC      =	2,
+      ONCORE_ANTENNA_NV      =	3
+};
+
 /* Model Name, derived from the @@Cj message.
  * Used to initialize some variables.
  */
@@ -212,6 +220,7 @@ struct instance {
 	enum receive_state o_state;		/* Receive state */
 	enum posn_mode mode;			/* 0D, 2D, 3D */
 	enum site_survey_state site_survey;	/* Site Survey state */
+	enum antenna_state ant_state;		/* antenna state */
 
 	int	Bj_day;
 
@@ -253,6 +262,7 @@ struct instance {
 	u_char	Ea[160];	/* Ba, Ea or Ha */
 	u_char	En[70]; 	/* Bn or En */
 	u_char	Cj[300];
+	u_char	Ag;		/* Satellite mask angle */
 	u_char	As;
 	u_char	Ay;
 	u_char	Az;
@@ -262,6 +272,7 @@ struct instance {
 	u_int	timeout;	/* count to retry Cj after Fa self-test */
 	u_char	count;		/* cycles thru Ea before starting */
 	s_char	assert;
+	u_char	hardpps;
 	u_int	saw_At;
 };
 
@@ -272,14 +283,18 @@ static	void	oncore_consume	     P((struct instance *));
 static	void	oncore_poll	     P((int, struct peer *));
 static	void	oncore_read_config   P((struct instance *));
 static	void	oncore_receive	     P((struct recvbuf *));
-static	void	oncore_sendmsg	     P((int fd, u_char *, size_t));
+static	void	oncore_sendmsg	     P((int, u_char *, size_t));
 static	void	oncore_shutdown      P((int, struct peer *));
 static	int	oncore_start	     P((int, struct peer *));
+static	void	oncore_control	     P((int, struct refclockstat *, struct refclockstat *, struct peer *));
+static	int	oncore_ppsapi	     P((struct instance *));
 static	void	oncore_get_timestamp P((struct instance *, long, long));
 static	void	oncore_init_shmem    P((struct instance *));
 static	void	oncore_print_As      P((struct instance *));
+static	void	oncore_check_antenna P((struct instance *, enum antenna_state));
 
 static	void	oncore_msg_any	   P((struct instance *, u_char *, size_t, int));
+static	void	oncore_msg_Ag	   P((struct instance *, u_char *, size_t));
 static	void	oncore_msg_As	   P((struct instance *, u_char *, size_t));
 static	void	oncore_msg_At	   P((struct instance *, u_char *, size_t));
 static	void	oncore_msg_Ay	   P((struct instance *, u_char *, size_t));
@@ -300,7 +315,7 @@ struct	refclock refclock_oncore = {
 	oncore_start,		/* start up driver */
 	oncore_shutdown,	/* shut down driver */
 	oncore_poll,		/* transmit poll message */
-	noentry,		/* not used */
+	oncore_control, 	/* fudge (flag) control messages */
 	noentry,		/* not used */
 	noentry,		/* not used */
 	NOFLAGS 		/* not used */
@@ -329,6 +344,7 @@ static struct msg_desc {
 	{ "Ad",  11,    0,                 "" },
 	{ "Ae",  11,    0,                 "" },
 	{ "Af",  15,    0,                 "" },
+	{ "Ag",   8,    oncore_msg_Ag,     "" }, /* Satellite mask angle */
 	{ "As",  20,    oncore_msg_As,     "" },
 	{ "At",   8,    oncore_msg_At,     "" },
 	{ "Au",  12,    0,                 "" },
@@ -386,6 +402,12 @@ u_char oncore_cmd_Gd0[] = { 'G', 'd', 0 };	/* 3D */
 u_char oncore_cmd_Gd1[] = { 'G', 'd', 1 };	/* 0D */
 u_char oncore_cmd_Gd2[] = { 'G', 'd', 2 };	/* 2D */
 
+/*
+ * Satellite mask angle.
+ */
+u_char oncore_cmd_Ag[] = { 'A', 'g', 0 };
+u_char oncore_cmd_Agx[] = { 'A', 'g', 0xff };
+
 /*
  * Set to UTC time (not GPS).
  */
@@ -502,10 +524,6 @@ static u_char oncore_cmd_Ia[] = { 'I', 'a' };	/* 12 Chan */
 	/* from buffer, char *buf, result to an int */
 #define buf_w32(buf) (((buf)[0]&0200) ? (-(~w32(buf)+1)) : w32(buf))
 
-extern int pps_assert;
-extern int pps_hardpps;
-
-
 
 /*
  * oncore_start - initialize data for processing
@@ -519,7 +537,7 @@ oncore_start(
 {
 	register struct instance *instance;
 	struct refclockproc *pp;
-	int fd1, fd2, mode;
+	int fd1, fd2;
 	char device1[30], device2[30];
 	const char *cp;
 	struct stat stat1, stat2;
@@ -548,6 +566,14 @@ oncore_start(
 		exit(1);
 	}
 
+	/* create instance structure for this unit */
+
+	if (!(instance = (struct instance *) malloc(sizeof *instance))) {
+		perror("malloc");
+		return (0);
+	}
+	memset((char *) instance, 0, sizeof *instance);
+
 	if ((stat1.st_dev == stat2.st_dev) && (stat1.st_ino == stat2.st_ino)) {
 		/* same device here */
 		if (!(fd1 = refclock_open(device1, SPEED, LDISC_RAW
@@ -559,7 +585,7 @@ oncore_start(
 			exit(1);
 		}
 		fd2 = fd1;
-	} else { /* different devices here */
+	} else {			/* different devices here */
 		if (!(fd1=refclock_open(device1, SPEED, LDISC_RAW))) {
 			perror("ONCORE: fd1");
 			exit(1);
@@ -570,24 +596,14 @@ oncore_start(
 		}
 	}
 
-	/* Devices now open, create instance structure for this unit */
-
-	if (!(instance = (struct instance *) malloc(sizeof *instance))) {
-		perror("malloc");
-		close(fd1);
-		return (0);
-	}
-	memset((char *) instance, 0, sizeof *instance);
-
-	/* link instance up and down */
+	/* initialize miscellaneous variables */
 
 	pp = peer->procptr;
 	pp->unitptr    = (caddr_t) instance;
 	instance->pp   = pp;
 	instance->unit = unit;
 	instance->peer = peer;
-
-	/* initialize miscellaneous variables */
+	instance->assert = 1;
 
 	instance->o_state = ONCORE_NO_IDEA;
 	cp = "state = ONCORE_NO_IDEA";
@@ -597,11 +613,11 @@ oncore_start(
 	instance->ppsfd = fd2;
 
 	instance->Bj_day = -1;
-	instance->assert = pps_assert;
 	instance->traim = -1;
 	instance->model = ONCORE_UNKNOWN;
 	instance->mode = MODE_UNKNOWN;
 	instance->site_survey = ONCORE_SS_UNKNOWN;
+	instance->Ag = 0xff;		/* Satellite mask angle, unset by user */
 
 	peer->precision = -26;
 	peer->minpoll = 4;
@@ -609,7 +625,7 @@ oncore_start(
 	pp->clockdesc = "Motorola Oncore GPS Receiver";
 	memcpy((char *)&pp->refid, "GPS\0", (size_t) 4);
 
-	/* go read any input data in /etc/ntp.oncoreX */
+	/* go read any input data in /etc/ntp.oncoreX or /etc/ntp/oncore.X */
 
 	oncore_read_config(instance);
 
@@ -619,66 +635,19 @@ oncore_start(
 		return(0);
 	}
 
-	if (time_pps_getcap(instance->pps_h, &mode) < 0) {
-		msyslog(LOG_ERR,
-		    "refclock_ioctl: time_pps_getcap failed: %m");
-		return (0);
+	if (instance->assert)
+		cp = "Initializing timing to Assert.";
+	else
+		cp = "Initializing timing to Clear.";
+	record_clock_stats(&(instance->peer->srcadr), cp);
+
+	if (instance->hardpps) {
+		cp = "HARDPPS Set.";
+		record_clock_stats(&(instance->peer->srcadr), cp);
 	}
 
-	if (time_pps_getparams(instance->pps_h, &instance->pps_p) < 0) {
-		msyslog(LOG_ERR,
-		    "refclock_ioctl: time_pps_getparams failed: %m");
-		return (0);
-	}
-
-	/* nb. only turn things on, if someone else has turned something
-	 *	on before we get here, leave it alone!
-	 */
-
-	if (instance->assert) { 	/* nb, default or ON */
-		instance->pps_p.mode = PPS_CAPTUREASSERT | PPS_OFFSETASSERT;
-		instance->pps_p.assert_offset.tv_sec = 0;
-		instance->pps_p.assert_offset.tv_nsec = 0;
-	} else {
-		instance->pps_p.mode = PPS_CAPTURECLEAR  | PPS_OFFSETCLEAR;
-		instance->pps_p.clear_offset.tv_sec = 0;
-		instance->pps_p.clear_offset.tv_nsec = 0;
-	}
-	instance->pps_p.mode |= PPS_TSFMT_TSPEC;
-	instance->pps_p.mode &= mode;		/* only set what is legal */
-
-	if (time_pps_setparams(instance->pps_h, &instance->pps_p) < 0) {
-		perror("time_pps_setparams");
-		exit(1);
-	}
-
-	if (pps_device && pps_device[0]) {
-		if (stat(pps_device, &stat1)) {
-			perror("ONCORE: stat pps_device");
-			return(0);
-		}
-
-		/* must have hardpps ON, and fd2 must be the same device as on the pps line */
-
-		if (pps_hardpps && ((stat1.st_dev == stat2.st_dev) && (stat1.st_ino == stat2.st_ino))) {
-			int	i;
-
-			if (instance->assert)
-				i = PPS_CAPTUREASSERT;
-			else
-				i = PPS_CAPTURECLEAR;
-
-			if (i&mode) {
-				if (time_pps_kcbind(instance->pps_h, PPS_KC_HARDPPS, i,
-				    PPS_TSFMT_TSPEC) < 0) {
-					msyslog(LOG_ERR,
-					    "refclock_ioctl: time_pps_kcbind failed: %m");
-					return (0);
-				}
-				pps_enable = 1;
-			}
-		}
-	}
+	if (!oncore_ppsapi(instance))
+		return(0);
 #endif
 
 	pp->io.clock_recv = oncore_receive;
@@ -707,6 +676,109 @@ oncore_start(
 }
 
 
+/*
+ * Fudge control (get Flag2 and Flag 3, not available at oncore_start time.
+ */
+
+static void
+oncore_control(
+	int unit,		  /* unit (not used */
+	struct refclockstat *in,  /* input parameters  (not uded) */
+	struct refclockstat *out, /* output parameters (not used) */
+	struct peer *peer	  /* peer structure pointer */
+	)
+{
+	char *cp;
+	struct refclockproc *pp;
+	struct instance *instance;
+
+	pp = peer->procptr;
+	instance = (struct instance *) pp->unitptr;
+
+	instance->assert  = !(pp->sloppyclockflag & CLK_FLAG2);
+	instance->hardpps =   pp->sloppyclockflag & CLK_FLAG3;
+
+	if (instance->assert)
+		cp = "Resetting timing to Assert.";
+	else
+		cp = "Resetting timing to Clear.";
+	record_clock_stats(&(instance->peer->srcadr), cp);
+
+	if (instance->hardpps) {
+		cp = "HARDPPS Set.";
+		record_clock_stats(&(instance->peer->srcadr), cp);
+	}
+
+	(void) oncore_ppsapi(instance);
+}
+
+
+/*
+ * Initialize PPSAPI
+ */
+
+#ifdef HAVE_PPSAPI
+static int
+oncore_ppsapi(
+	struct instance *instance
+	)
+{
+	int mode;
+
+	if (time_pps_getcap(instance->pps_h, &mode) < 0) {
+		msyslog(LOG_ERR, "refclock_ioctl: time_pps_getcap failed: %m");
+		return (0);
+	}
+
+	if (time_pps_getparams(instance->pps_h, &instance->pps_p) < 0) {
+		msyslog(LOG_ERR, "refclock_ioctl: time_pps_getparams failed: %m");
+		return (0);
+	}
+
+	/* nb. only turn things on, if someone else has turned something
+	 *	on before we get here, leave it alone!
+	 */
+
+	if (instance->assert) { 	/* nb, default or ON */
+		instance->pps_p.mode = PPS_CAPTUREASSERT | PPS_OFFSETASSERT;
+		instance->pps_p.assert_offset.tv_sec = 0;
+		instance->pps_p.assert_offset.tv_nsec = 0;
+	} else {
+		instance->pps_p.mode = PPS_CAPTURECLEAR  | PPS_OFFSETCLEAR;
+		instance->pps_p.clear_offset.tv_sec = 0;
+		instance->pps_p.clear_offset.tv_nsec = 0;
+	}
+	instance->pps_p.mode |= PPS_TSFMT_TSPEC;
+	instance->pps_p.mode &= mode;		/* only set what is legal */
+
+	if (time_pps_setparams(instance->pps_h, &instance->pps_p) < 0) {
+		perror("time_pps_setparams");
+		exit(1);
+	}
+
+	/* must have hardpps ON */
+
+	if (instance->hardpps) {
+		int	i;
+
+		if (instance->assert)
+			i = PPS_CAPTUREASSERT;
+		else
+			i = PPS_CAPTURECLEAR;
+
+		if (i&mode) {
+			if (time_pps_kcbind(instance->pps_h, PPS_KC_HARDPPS, i,
+			    PPS_TSFMT_TSPEC) < 0) {
+				msyslog(LOG_ERR, "refclock_ioctl: time_pps_kcbind failed: %m");
+				return (0);
+			}
+			pps_enable = 1;
+		}
+	}
+	return(1);
+}
+#endif
+
 
 /*
  * Read Input file if it exists.
@@ -751,8 +823,8 @@ oncore_read_config(
  * -------------------------------------------------------------------------------
  *
  * If we open one or the other of the files, we read it looking for
- *   MODE, LAT, LON, (HT, HTGPS, HTMSL), DELAY, OFFSET, ASSERT, CLEAR, STATUS,
- *   POSN3D, POSN2D, CHAN, TRAIM
+ *   MODE, LAT, LON, (HT, HTGPS, HTMSL), DELAY, OFFSET, ASSERT, CLEAR, HARDPPS,
+ *   STATUS, POSN3D, POSN2D, CHAN, TRAIM
  * then initialize using method MODE.  For Mode = (1,3) all of (LAT, LON, HT) must
  *   be present or mode reverts to (2,4).
  *
@@ -793,17 +865,24 @@ oncore_read_config(
  *	There is an optional line, with either ASSERT or CLEAR on it, which
  *	   determine which transition of the PPS signal is used for timing by the
  *	   PPSAPI.  If neither is present, then ASSERT is assumed.
+ *	   ASSERT/CLEAR can also be set with FLAG2 of the ntp.conf input.
+ *	   For Flag2, ASSERT=0, and hence is default.
+ *
+ *	There is an optional line, with HARDPPS on it.	Including this line causes
+ *	   the PPS signal to control the kernel PLL.
+ *	   HARDPPS can also be set with FLAG3 of the ntp.conf input.
+ *	   For Flag3, 0 is disabled, and the default.
  *
  *	There are three options that have to do with using the shared memory opition.
- *	   First, to enable the option there must be an ASSERT line with a file name.
+ *	   First, to enable the option there must be a SHMEM line with a file name.
  *	   The file name is the file associated with the shared memory.
  *
- *	In the shared memory there are three 'records' containing the @@Ea (or equivalent)
- *	   data, and this contains the position data.  There will always be data in the
- *	   record cooresponding to the '0D' @@Ea record, and the user has a choice of
- *	   filling the '3D' @@Ea record by specifying POSN3D, or the '2D' record by
- *	   specifying POSN2D.  In either case the '2D' or '3D' record is filled once
- *	   every 15s.
+ *	In shared memory, there is one 'record' for each returned variable.
+ *	For the @@Ea data there are three 'records' containing position data.
+ *	   There will always be data in the record cooresponding to the '0D' @@Ea record,
+ *	   and the user has a choice of filling the '3D' record by specifying POSN3D,
+ *	   or the '2D' record by specifying POSN2D.  In either case the '2D' or '3D'
+ *	   record is filled once every 15s.
  *
  *	Two additional variables that can be set are CHAN and TRAIM.  These should be
  *	   set correctly by the code examining the @@Cj record, but we bring them out here
@@ -811,6 +890,11 @@ oncore_read_config(
  *	   CHAN expects to be followed by in integer: 6, 8, or 12. TRAIM expects to be
  *	   followed by YES or NO.
  *
+ *	There is an optional line with MASK on it followed by one integer field in the
+ *	   range 0 to 89. This sets the satellite mask angle and will determine the minimum
+ *	   elevation angle for sattelites to be tracked by the receiver. The default value
+ *	   is 10 deg for the VP and 0 deg for all other receivers.
+ *
  * So acceptable input would be
  *	# these are my coordinates (RWC)
  *	LON  -106 34.610
@@ -821,7 +905,7 @@ oncore_read_config(
 
 	FILE	*fd;
 	char	*cp, *cc, *ca, line[100], units[2], device[20], Msg[160];
-	int	i, sign, lat_flg, long_flg, ht_flg, mode;
+	int	i, sign, lat_flg, long_flg, ht_flg, mode, mask;
 	double	f1, f2, f3;
 
 	sprintf(device, "%s%d", INIT_FILE, instance->unit);             /* try "ntp.oncore0" first */
@@ -835,7 +919,7 @@ oncore_read_config(
 		}
 	}
 
-	mode = 0;
+	mode = mask = 0;
 	lat_flg = long_flg = ht_flg = 0;
 	while (fgets(line, 100, fd)) {
 
@@ -963,6 +1047,8 @@ oncore_read_config(
 			instance->assert = 1;
 		} else if (!strncmp(cc, "CLEAR", (size_t) 5)) {
 			instance->assert = 0;
+		} else if (!strncmp(cc, "HARDPPS", (size_t) 7)) {
+			instance->hardpps = 1;
 		} else if (!strncmp(cc, "POSN2D", (size_t) 6)) {
 			instance->shmem_Posn = 2;
 		} else if (!strncmp(cc, "POSN3D", (size_t) 6)) {
@@ -975,6 +1061,10 @@ oncore_read_config(
 			instance->traim = 1;				/* so TRAIM alone is YES */
 			if (!strcmp(ca, "NO") || !strcmp(ca, "OFF"))    /* Yes/No, On/Off */
 				instance->traim = 0;
+		} else if (!strncmp(cc, "MASK", (size_t) 4)) {
+			sscanf(ca, "%d", &mask);
+			if (mask > -1 && mask < 90)
+				instance->Ag = mask;			/* Satellite mask angle */
 		}
 	}
 	fclose(fd);
@@ -1053,7 +1143,7 @@ oncore_init_shmem(
 		perror("malloc");
 		return;
 	}
-	memset(buf, 0, sizeof(buf));
+	memset(buf, 0, oncore_shmem_length);
 	i = write(instance->statusfd, buf, oncore_shmem_length);
 	if (i != oncore_shmem_length) {
 		perror(instance->shmem_fname);
@@ -1336,9 +1426,9 @@ oncore_msg_any(
 
 	if (debug > 3) {
 #ifdef HAVE_GETCLOCK
-		(void) getclock(TIMEOFDAY, &ts); 
-		tv.tv_sec = ts.tv_sec; 
-		tv.tv_usec = ts.tv_nsec / 1000; 
+		(void) getclock(TIMEOFDAY, &ts);
+		tv.tv_sec = ts.tv_sec;
+		tv.tv_usec = ts.tv_nsec / 1000;
 #else
 		GETTIMEOFDAY(&tv, 0);
 #endif
@@ -1487,10 +1577,10 @@ oncore_msg_Cj_id(
 	} else if (Model[0] == 'B' || !strncmp(Model, "T8", (size_t) 2)) {
 		cp = "VP";
 		instance->model = ONCORE_VP;
-	} else if (!strncmp(Model, "P1", (size_t) 2)) {
+	} else if (Model[0] == 'P') {
 		cp = "M12";
 		instance->model = ONCORE_M12;
-	} else if (Model[0] == 'R') {
+	} else if (Model[0] == 'R' || Model[0] == 'D' || Model[0] == 'S') {
 		if (Model[5] == 'N') {
 			cp = "GT";
 			instance->model = ONCORE_GT;
@@ -1559,7 +1649,7 @@ oncore_msg_Cj_id(
 	 * instance->shmem_fname if SHMEM is not supported
 	 */
 
-	if (instance->shmem_fname);
+	if (instance->shmem_fname)
 		oncore_init_shmem(instance);
 
 	if (instance->shmem)
@@ -1568,14 +1658,6 @@ oncore_msg_Cj_id(
 		cp = "SHMEM is NOT available";
 	record_clock_stats(&(instance->peer->srcadr), cp);
 
-#ifdef HAVE_PPSAPI
-	if (instance->assert)
-		cp = "Timing on Assert.";
-	else
-		cp = "Timing on Clear.";
-	record_clock_stats(&(instance->peer->srcadr), cp);
-#endif
-
 	mode = instance->init_type;
 	if (mode == 3 || mode == 4) {	/* Cf will call Fa */
 		oncore_sendmsg(instance->ttyfd, oncore_cmd_Cf, sizeof(oncore_cmd_Cf));
@@ -1733,6 +1815,14 @@ oncore_msg_Cj_init(
 				instance->offset = 0;
 			}
 		}
+
+		/* Satellite mask angle */
+
+		if (instance->Ag != 0xff) {	/* will have 0xff in it if not set by user */
+			memcpy(Cmd, oncore_cmd_Ag, sizeof(oncore_cmd_Ag));
+			Cmd[2] = instance->Ag;
+			oncore_sendmsg(instance->ttyfd, Cmd,  sizeof(oncore_cmd_Ag));
+		}
 	}
 
 	/* 6, 8 12 chan - Position/Status/Data Output Message, 1/s */
@@ -1812,7 +1902,7 @@ oncore_msg_CaFaIa(
 	char *cp;
 
 	if (instance->o_state == ONCORE_TEST_SENT) {
-		int	antenna;
+		enum antenna_state antenna;
 
 		instance->timeout = 0;
 
@@ -1823,27 +1913,30 @@ oncore_msg_CaFaIa(
 				printf("ONCORE[%d]: >>@@%ca %x %x\n", instance->unit, buf[2], buf[4], buf[5]);
 		}
 
-		antenna = buf[4] & 0xc0;
-		antenna >>= 6;
+		antenna = (buf[4] & 0xc0) >> 6;
 		buf[4] &= ~0xc0;
 
 		if (buf[4] || buf[5] || ((buf[2] == 'I') && buf[6])) {
-			cp = "ONCORE: Self Test Failed, shutting down driver";
-			record_clock_stats(&(instance->peer->srcadr), cp);
+			if (buf[2] == 'I') {
+				msyslog(LOG_ERR, "ONCORE[%d]: self test failed: result %02x %02x %02x",
+					instance->unit, buf[4], buf[5], buf[6]);
+			} else {
+				msyslog(LOG_ERR, "ONCORE[%d]: self test failed: result %02x %02x",
+					instance->unit, buf[4], buf[5]);
+			}
+			cp = "ONCORE: self test failed, shutting down driver";
+			record_clock_stats(&instance->peer->srcadr, cp);
+
+			refclock_report(instance->peer, CEVNT_FAULT);
+
+
 			oncore_shutdown(instance->unit, instance->peer);
 			return;
 		}
-		if (antenna) {
-			char *cp1, Msg[160];
 
-			cp1 = (antenna == 0x1) ? "(Over Current)" :
-				((antenna == 0x2) ? "(Under Current)" : "(No Voltage)");
+		/* report the current antenna state */
 
-			cp = "ONCORE: Self Test, NonFatal Antenna Problems ";
-			strcpy(Msg, cp);
-			strcat(Msg, cp1);
-			record_clock_stats(&(instance->peer->srcadr), Msg);
-		}
+		oncore_check_antenna(instance, antenna);
 
 		oncore_sendmsg(instance->ttyfd, oncore_cmd_Cj, sizeof(oncore_cmd_Cj));
 		instance->o_state = ONCORE_INIT;
@@ -1867,6 +1960,7 @@ oncore_msg_BaEaHa(
 	char		Msg[160], Cmd[20];
 	u_char		*vp;	/* pointer to start of shared mem for Ba/Ea/Ha */
 	size_t		Len;
+	enum antenna_state antenna;
 
 	/* At the beginning of Ea here there are various 'timers'.
 	 * We enter Ea 1/sec, and since the upper levels of NTP have usurped
@@ -1883,7 +1977,7 @@ oncore_msg_BaEaHa(
 	if (instance->o_state != ONCORE_ALMANAC && instance->o_state != ONCORE_RUN)
 		return;
 
-	Len = len+3;		/* message length @@ -> CR,LF */
+	Len = len+3;		/* message length @@ -> checksum, CR, LF */
 	memcpy(instance->Ea, buf, Len); 	/* Ba, Ea or Ha */
 
 	if (buf[2] == 'B') {			/* 6chan */
@@ -1913,14 +2007,17 @@ oncore_msg_BaEaHa(
 	}
 
 	vp = (u_char) 0;	/* just to keep compiler happy */
+	antenna = instance->ant_state;
 	if (instance->chan == 6) {
 		instance->rsm.bad_almanac = instance->Ea[64]&0x1;
 		instance->rsm.bad_fix	  = instance->Ea[64]&0x52;
 		vp = &instance->shmem[instance->shmem_Ba];
+		antenna = (instance->Ea[37] & 0xc0) >> 6;  /* prob unset */
 	} else if (instance->chan == 8) {
 		instance->rsm.bad_almanac = instance->Ea[72]&0x1;
 		instance->rsm.bad_fix	  = instance->Ea[72]&0x52;
 		vp = &instance->shmem[instance->shmem_Ea];
+		antenna = (instance->Ea[37] & 0xc0) >> 6;  /* set GT, UT, unset VP */
 	} else if (instance->chan == 12) {
 		int bits1, bits2;
 
@@ -1930,6 +2027,7 @@ oncore_msg_BaEaHa(
 		instance->rsm.bad_fix	  = (bits2 & 0x8) || (bits1 == 0x2);
 					  /* too few sat     Bad Geom	  */
 		vp = &instance->shmem[instance->shmem_Ha];
+		antenna = (bits2 & 0x6 ) >> 1;
 #if 0
 fprintf(stderr, "ONCORE: DEBUG BITS: (%x %x), (%x %x),  %x %x %x %x %x\n",
 		instance->Ea[129], instance->Ea[130], bits1, bits2, instance->mode == MODE_0D, instance->mode == MODE_2D,
@@ -2081,6 +2179,9 @@ fprintf(stderr, "ONCORE: DEBUG BITS: (%x %x), (%x %x),  %x %x %x %x %x\n",
 
 			/* Read back Cable Delay for Output */
 		oncore_sendmsg(instance->ttyfd, oncore_cmd_Azx,  sizeof(oncore_cmd_Azx));
+
+			/* Read back Satellite Mask Angle for Output */
+		oncore_sendmsg(instance->ttyfd, oncore_cmd_Agx,  sizeof(oncore_cmd_Agx));
 	}
 
 	/*
@@ -2252,8 +2353,8 @@ oncore_get_timestamp(
 	long dt2	/* tick offset NEXT time step */
 	)
 {
-	int     Rsm;
-	u_long  i, j;
+	int	Rsm;
+	u_long	i, j;
 	l_fp ts, ts_tmp;
 	double dmy;
 #ifdef HAVE_STRUCT_TIMESPEC
@@ -2431,14 +2532,12 @@ oncore_get_timestamp(
 	 */
 
 	if (time_pps_getcap(instance->pps_h, &current_mode) < 0) {
-		msyslog(LOG_ERR,
-		    "refclock_ioctl: time_pps_getcap failed: %m");
+		msyslog(LOG_ERR, "refclock_ioctl: time_pps_getcap failed: %m");
 		return;
 	}
 
 	if (time_pps_getparams(instance->pps_h, &current_params) < 0) {
-		msyslog(LOG_ERR,
-		    "refclock_ioctl: time_pps_getparams failed: %m");
+		msyslog(LOG_ERR, "refclock_ioctl: time_pps_getparams failed: %m");
 		return;
 	}
 
@@ -2466,7 +2565,6 @@ oncore_get_timestamp(
 
 	ts.l_ui += JAN_1970;
 	instance->pp->lastrec = ts;
-	instance->pp->msec = 0;
 
 	ts_tmp = ts;
 	ts_tmp.l_ui = 0;	/* zero integer part */
@@ -2533,6 +2631,7 @@ oncore_get_timestamp(
 /*
 		instance->pp->dispersion = instance->pp->skew = 0;
 */
+		instance->pp->lastref = instance->pp->lastrec;
 		refclock_receive(instance->peer);
 	}
 }
@@ -2562,6 +2661,24 @@ oncore_msg_At(
 }
 
 
+static void
+oncore_msg_Ag(
+	struct instance *instance,
+	u_char *buf,
+	size_t len
+	)
+{		char  Msg[160], *cp;
+
+		cp = "set to";
+		if (instance->o_state == ONCORE_RUN)
+			cp = "is";
+
+		instance->Ag = buf[4];
+		sprintf(Msg, "Satellite mask angle %s %d degrees", cp, (int) instance->Ag);
+		record_clock_stats(&(instance->peer->srcadr), Msg);
+}
+
+
 
 /* get leap-second warning message */
 
@@ -2803,6 +2920,26 @@ oncore_msg_Sz(
 	}
 }
 
+
+static void
+oncore_check_antenna(
+	struct instance *instance,
+	enum antenna_state new_state)
+{
+	char *cp;
+
+	switch (new_state) {
+	case ONCORE_ANTENNA_OK: cp = "GPS antenna: OK";                   break;
+	case ONCORE_ANTENNA_OC: cp = "GPS antenna: short (overcurrent)";  break;
+	case ONCORE_ANTENNA_UC: cp = "GPS antenna: open (not connected)"; break;
+	case ONCORE_ANTENNA_NV: cp = "GPS antenna: short (no voltage)";   break;
+	default:		cp = "GPS antenna: ?";                    break;
+	}
+
+	instance->ant_state = new_state;
+	record_clock_stats(&instance->peer->srcadr, cp);
+}
+
 #else
 int refclock_oncore_bs;
 #endif /* REFCLOCK */
