@@ -14,6 +14,8 @@
 #include <ctype.h>
 #include <signal.h>
 #include <setjmp.h>
+#include <sys/types.h>
+#include <sys/time.h>
 #include <netdb.h>
 #ifdef SYS_WINNT
 # include <io.h>
@@ -165,7 +167,6 @@ struct ctl_var peer_var[] = {
 	{ CP_FILTERROR,	AR,	"filtdisp" },	/* 34 */
 	{ CP_FLASH,     FX,	"flash" },	/* 35 */ 
 	{ CP_TTL,	UI,	"ttl" },	/* 36 */
-	{ CP_TTLMAX,	UI,	"ttlmax" },	/* 37 */
 	/*
 	 * These are duplicate entries so that we can
 	 * process deviant version of the ntp protocol.
@@ -291,8 +292,8 @@ struct xcmd builtins[] = {
 	{ "delay",	auth_delay,	{ OPT|INT, NO, NO, NO },
 	  { "msec", "", "", "" },
 	  "set the delay added to encryption time stamps" },
-	{ "host",	host,		{ OPT|STR, NO, NO, NO },
-	  { "hostname", "", "", "" },
+	{ "host",	host,		{ OPT|STR, OPT|STR, NO, NO },
+	  { "-4|-6", "hostname", "", "" },
 	  "specify the host whose NTP server we talk to" },
 	{ "poll",	ntp_poll,	{ OPT|UINT, OPT|STR, NO, NO },
 	  { "n", "verbose", "", "" },
@@ -344,7 +345,7 @@ struct xcmd builtins[] = {
 #define	DEFTIMEOUT	(5)		/* 5 second time out */
 #define	DEFSTIMEOUT	(2)		/* 2 second time out after first */
 #define	DEFDELAY	0x51EB852	/* 20 milliseconds, l_fp fraction */
-#define	DEFHOST		"127.0.0.1"	/* default host name */
+#define	DEFHOST		"localhost"	/* default host name */
 #define	LENHOSTNAME	256		/* host name is 256 characters long */
 #define	MAXCMDS		100		/* maximum commands on cmd line */
 #define	MAXHOSTS	200		/* maximum hosts on cmd line */
@@ -364,8 +365,10 @@ char currenthost[LENHOSTNAME];			/* current host name */
 struct sockaddr_in hostaddr = { 0 };		/* host address */
 int showhostnames = 1;				/* show host names by default */
 
-int sockfd;					/* fd socket is opened on */
+int ai_fam_templ;				/* address family */
+SOCKET sockfd;					/* fd socket is opened on */
 int havehost = 0;				/* set to 1 when host open */
+int s_port = 0;
 struct servent *server_entry = NULL;		/* server entry for ntp */
 
 #ifdef SYS_WINNT
@@ -496,8 +499,14 @@ ntpqmain(
 	delay_time.l_uf = DEFDELAY;
 
 	progname = argv[0];
-	while ((c = ntp_getopt(argc, argv, "c:dinp")) != EOF)
+	while ((c = ntp_getopt(argc, argv, "46c:dinp")) != EOF)
 	    switch (c) {
+		case '4':
+		    ai_fam_templ = AF_INET;
+		    break;
+		case '6':
+		    ai_fam_templ = AF_INET6;
+		    break;
 		case 'c':
 		    ADDCMD(ntp_optarg);
 		    break;
@@ -519,7 +528,7 @@ ntpqmain(
 	    }
 	if (errflg) {
 		(void) fprintf(stderr,
-			       "usage: %s [-dinp] [-c cmd] host ...\n",
+			       "usage: %s [-46dinp] [-c cmd] host ...\n",
 			       progname);
 		exit(2);
 	}
@@ -576,29 +585,69 @@ openhost(
 	const char *hname
 	)
 {
-	u_int32 netnum;
 	char temphost[LENHOSTNAME];
+	int a_info, i;
+	struct addrinfo hints, *ai = NULL;
+	register const char *cp;
+	char name[LENHOSTNAME];
+	char service[5];
 
-	if (server_entry == NULL) {
-		server_entry = getservbyname("ntp", "udp");
-		if (server_entry == NULL) {
-#ifdef VMS /* UCX getservbyname() doesn't work [yet], but we do know better */
-			server_entry = (struct servent *)
-				malloc(sizeof(struct servent));
-			server_entry->s_port = htons(NTP_PORT);
-#else
-			(void) fprintf(stderr, "%s: ntp/udp: unknown service\n",
-				       progname);
-			exit(1);
-#endif /* VMS & UCX */
-		}
-		if (debug > 2)
-		    printf("Got ntp/udp service entry\n");
+	/*
+	 * We need to get by the [] if they were entered
+	 */
+	
+	cp = hname;
+	
+	if(*cp == '[') {
+		cp++;
+		for(i = 0; *cp != ']'; cp++, i++)
+			name[i] = *cp;
+	name[i] = '\0';
+	hname = name;
+	}
+
+	/*
+	 * First try to resolve it as an ip address and if that fails,
+	 * do a fullblown (dns) lookup. That way we only use the dns
+	 * when it is needed and work around some implementations that
+	 * will return an "IPv4-mapped IPv6 address" address if you
+	 * give it an IPv4 address to lookup.
+	 */
+	sprintf(service, "%u", NTP_PORT);
+	memset((char *)&hints, 0, sizeof(struct addrinfo));
+	hints.ai_family = ai_fam_templ;
+	hints.ai_protocol = IPPROTO_UDP;
+	hints.ai_socktype = SOCK_DGRAM;
+	hints.ai_flags = AI_NUMERICHOST;
+
+	a_info = getaddrinfo(hname, service, &hints, &ai);
+	if (a_info == EAI_NONAME) {
+		hints.ai_flags = AI_CANONNAME;
+#ifdef AI_ADDRCONFIG
+		hints.ai_flags |= AI_ADDRCONFIG;
+#endif
+		a_info = getaddrinfo(hname, service, &hints, &ai);	
+	}
+	/* Some older implementations don't like AI_ADDRCONFIG. */
+	if (a_info == EAI_BADFLAGS) {
+		hints.ai_flags = AI_CANONNAME;
+		a_info = getaddrinfo(hname, service, &hints, &ai);	
+	}
+	if (a_info != 0) {
+		(void) fprintf(stderr, "%s\n", gai_strerror(a_info));
+		return 0;
+	}
+
+	if (ai->ai_canonname == NULL) {
+		strncpy(temphost, stoa((struct sockaddr_storage *)ai->ai_addr),
+		    LENHOSTNAME);
+		temphost[LENHOSTNAME-1] = '\0';
+
+	} else {
+		strncpy(temphost, ai->ai_canonname, LENHOSTNAME);
+		temphost[LENHOSTNAME-1] = '\0';
 	}
 
-	if (!getnetnum(hname, &netnum, temphost))
-	    return 0;
-	
 	if (debug > 2)
 	    printf("Opening host %s\n", temphost);
 
@@ -610,13 +659,17 @@ openhost(
 	}
 	(void) strcpy(currenthost, temphost);
 
-	hostaddr.sin_family = AF_INET;
-#ifndef SYS_VXWORKS
-	hostaddr.sin_port = server_entry->s_port;
-#else
-    hostaddr.sin_port = htons(SERVER_PORT_NUM);
-#endif
-	hostaddr.sin_addr.s_addr = netnum;
+	/* port maps to the same location in both families */
+	s_port = ((struct sockaddr_in6 *)ai->ai_addr)->sin6_port;
+#ifdef SYS_VXWORKS
+	((struct sockaddr_in6 *)&hostaddr)->sin6_port = htons(SERVER_PORT_NUM);
+	if (ai->ai_family == AF_INET)
+		*(struct sockaddr_in *)&hostaddr=
+			*((struct sockaddr_in *)ai->ai_addr);
+	else
+		*(struct sockaddr_in6 *)&hostaddr=
+			*((struct sockaddr_in6 *)ai->ai_addr);
+#endif /* SYS_VXWORKS */
 
 #ifdef SYS_WINNT
 	{
@@ -628,18 +681,12 @@ openhost(
 			exit(1);
 		}
 	}
+#endif /* SYS_WINNT */
 
-
-	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
+	sockfd = socket(ai->ai_family, SOCK_DGRAM, 0);
 	if (sockfd == INVALID_SOCKET) {
 		error("socket", "", "");
-		exit(-1);
 	}
-#else
-	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (sockfd == -1)
-	    error("socket", "", "");
-#endif /* SYS_WINNT */
 
 	
 #ifdef NEED_RCVBUF_SLOP
@@ -652,10 +699,16 @@ openhost(
 # endif
 #endif
 
+#ifdef SYS_VXWORKS
 	if (connect(sockfd, (struct sockaddr *)&hostaddr,
 		    sizeof(hostaddr)) == -1)
+#else
+	if (connect(sockfd, (struct sockaddr *)ai->ai_addr,
+		    ai->ai_addrlen) == -1)
+#endif /* SYS_VXWORKS */
 	    error("connect", "", "");
-	
+	if (a_info == 0)
+		freeaddrinfo(ai);
 	havehost = 1;
 	return 1;
 }
@@ -1068,7 +1121,7 @@ sendrequest(
 	 * Fill in the packet
 	 */
 	qpkt.li_vn_mode = PKT_LI_VN_MODE(0, pktversion, MODE_CONTROL);
-	qpkt.r_m_e_op = (u_char)opcode & CTL_OP_MASK;
+	qpkt.r_m_e_op = (u_char)(opcode & CTL_OP_MASK);
 	qpkt.sequence = htons(sequence);
 	qpkt.status = 0;
 	qpkt.associd = htons((u_short)associd);
@@ -1291,7 +1344,7 @@ getcmds(void)
 #endif /* not HAVE_LIBREADLINE */
 }
 
-
+#ifndef SYS_WINNT /* Under NT cannot handle SIGINT, WIN32 spawns a handler */
 /*
  * abortcmd - catch interrupts and abort the current command
  */
@@ -1306,7 +1359,7 @@ abortcmd(
 	(void) fflush(stderr);
 	if (jump) longjmp(interrupt_buf, 1);
 }
-
+#endif	/* SYS_WINNT */
 
 /*
  * docmd - decode the command line and execute a command
@@ -1519,7 +1572,7 @@ getarg(
 		argp->string = str;
 		break;
 	    case ADD:
-		if (!getnetnum(str, &(argp->netnum), (char *)0)) {
+		if (!getnetnum(str, &(argp->netnum), (char *)0, 0)) {
 			return 0;
 		}
 		break;
@@ -1571,6 +1624,17 @@ getarg(
 			argp->ival = -argp->ival;
 		}
 		break;
+	     case IP_VERSION:
+		if (!strcmp("-6", str))
+			argp->ival = 6 ;
+		else if (!strcmp("-4", str))
+			argp->ival = 4 ;
+		else {
+			(void) fprintf(stderr,
+			    "***Version must be either 4 or 6\n");
+			return 0;
+		}
+		break;
 	}
 
 	return 1;
@@ -1584,25 +1648,37 @@ getarg(
 int
 getnetnum(
 	const char *hname,
-	u_int32 *num,
-	char *fullhost
+	struct sockaddr_storage *num,
+	char *fullhost,
+	int af
 	)
 {
-	struct hostent *hp;
+	int err;
+	int sockaddr_len;
+	struct addrinfo hints, *ai = NULL;
 
+	sockaddr_len = (af == AF_INET)
+			   ? sizeof(struct sockaddr_in)
+			   : sizeof(struct sockaddr_in6);
+	memset((char *)&hints, 0, sizeof(struct addrinfo));
+	hints.ai_flags = AI_CANONNAME;
+#ifdef AI_ADDRCONFIG
+	hints.ai_flags |= AI_ADDRCONFIG;
+#endif
+	
+	/* decodenetnum works with addresses only */
 	if (decodenetnum(hname, num)) {
 		if (fullhost != 0) {
-			(void) sprintf(fullhost, "%lu.%lu.%lu.%lu",
-				       (u_long)((htonl(*num) >> 24) & 0xff),
-				       (u_long)((htonl(*num) >> 16) & 0xff),
-				       (u_long)((htonl(*num) >> 8) & 0xff),
-				       (u_long)(htonl(*num) & 0xff));
+			getnameinfo((struct sockaddr *)num, sockaddr_len,
+					fullhost, sizeof(fullhost), NULL, 0,
+					NI_NUMERICHOST);
+
 		}
 		return 1;
-	} else if ((hp = gethostbyname(hname)) != 0) {
-		memmove((char *)num, hp->h_addr, sizeof(u_int32));
+	} else if ((err = getaddrinfo(hname, "ntp", &hints, &ai)) == 0) {
+		memmove((char *)num, ai->ai_addr, ai->ai_addrlen);
 		if (fullhost != 0)
-		    (void) strcpy(fullhost, hp->h_name);
+		    (void) strcpy(fullhost, ai->ai_canonname);
 		return 1;
 	} else {
 		(void) fprintf(stderr, "***Can't find host %s\n", hname);
@@ -1617,14 +1693,14 @@ getnetnum(
  */
 char *
 nntohost(
-	u_int32 netnum
+	struct sockaddr_storage *netnum
 	)
 {
 	if (!showhostnames)
-	    return numtoa(netnum);
-	if ((ntohl(netnum) & REFCLOCK_MASK) == REFCLOCK_ADDR)
-	    return refnumtoa(netnum);
-	return numtohost(netnum);
+	    return stoa(netnum);
+	if ((netnum->ss_family == AF_INET) && ISREFCLOCKADR(netnum))
+    		return refnumtoa(netnum);
+	return socktohost(netnum);
 }
 
 
@@ -1669,10 +1745,10 @@ rtdatetolfp(
 		return 0;
 	}
 
-	cal.monthday = *cp++ - '0';	/* ascii dependent */
+	cal.monthday = (u_char) (*cp++ - '0');	/* ascii dependent */
 	if (isdigit((int)*cp)) {
-		cal.monthday = (cal.monthday << 3) + (cal.monthday << 1);
-		cal.monthday += *cp++ - '0';
+		cal.monthday = (u_char)((cal.monthday << 3) + (cal.monthday << 1));
+		cal.monthday = (u_char)(cal.monthday + *cp++ - '0');
 	}
 
 	if (*cp++ != '-')
@@ -1687,25 +1763,25 @@ rtdatetolfp(
 		break;
 	if (i == 12)
 	    return 0;
-	cal.month = i + 1;
+	cal.month = (u_char)(i + 1);
 
 	if (*cp++ != '-')
 	    return 0;
 	
 	if (!isdigit((int)*cp))
 	    return 0;
-	cal.year = *cp++ - '0';
+	cal.year = (u_short)(*cp++ - '0');
 	if (isdigit((int)*cp)) {
-		cal.year = (cal.year << 3) + (cal.year << 1);
-		cal.year += *cp++ - '0';
+		cal.year = (u_short)((cal.year << 3) + (cal.year << 1));
+		cal.year = (u_short)(*cp++ - '0');
 	}
 	if (isdigit((int)*cp)) {
-		cal.year = (cal.year << 3) + (cal.year << 1);
-		cal.year += *cp++ - '0';
+		cal.year = (u_short)((cal.year << 3) + (cal.year << 1));
+		cal.year = (u_short)(cal.year + *cp++ - '0');
 	}
 	if (isdigit((int)*cp)) {
-		cal.year = (cal.year << 3) + (cal.year << 1);
-		cal.year += *cp++ - '0';
+		cal.year = (u_short)((cal.year << 3) + (cal.year << 1));
+		cal.year = (u_short)(cal.year + *cp++ - '0');
 	}
 
 	/*
@@ -1718,26 +1794,26 @@ rtdatetolfp(
 
 	if (*cp++ != ' ' || !isdigit((int)*cp))
 	    return 0;
-	cal.hour = *cp++ - '0';
+	cal.hour = (u_char)(*cp++ - '0');
 	if (isdigit((int)*cp)) {
-		cal.hour = (cal.hour << 3) + (cal.hour << 1);
-		cal.hour += *cp++ - '0';
+		cal.hour = (u_char)((cal.hour << 3) + (cal.hour << 1));
+		cal.hour = (u_char)(cal.hour + *cp++ - '0');
 	}
 
 	if (*cp++ != ':' || !isdigit((int)*cp))
 	    return 0;
-	cal.minute = *cp++ - '0';
+	cal.minute = (u_char)(*cp++ - '0');
 	if (isdigit((int)*cp)) {
-		cal.minute = (cal.minute << 3) + (cal.minute << 1);
-		cal.minute += *cp++ - '0';
+		cal.minute = (u_char)((cal.minute << 3) + (cal.minute << 1));
+		cal.minute = (u_char)(cal.minute + *cp++ - '0');
 	}
 
 	if (*cp++ != ':' || !isdigit((int)*cp))
 	    return 0;
-	cal.second = *cp++ - '0';
+	cal.second = (u_char)(*cp++ - '0');
 	if (isdigit((int)*cp)) {
-		cal.second = (cal.second << 3) + (cal.second << 1);
-		cal.second += *cp++ - '0';
+		cal.second = (u_char)((cal.second << 3) + (cal.second << 1));
+		cal.second = (u_char)(cal.second + *cp++ - '0');
 	}
 
 	/*
@@ -1906,7 +1982,7 @@ help(
 	int n;
 	struct xcmd *xcp;
 	char *cmd;
-	const char *cmdsort[100];
+	char *cmdsort[100];
 	int length[100];
 	int maxlength;
 	int numperline;
@@ -1916,10 +1992,10 @@ help(
 		n = 0;
 		for (xcp = builtins; xcp->keyword != 0; xcp++) {
 			if (*(xcp->keyword) != '?')
-			    cmdsort[n++] = xcp->keyword;
+			    cmdsort[n++] = (char *)xcp->keyword;
 		}
 		for (xcp = opcmds; xcp->keyword != 0; xcp++)
-		    cmdsort[n++] = xcp->keyword;
+		    cmdsort[n++] = (char *)xcp->keyword;
 
 #ifdef QSORT_USES_VOID_P
 		qsort(cmdsort, (size_t)n, sizeof(char *), helpsort);
@@ -1973,8 +2049,8 @@ helpsort(
 	const void *t2
 	)
 {
-	const char **name1 = (const char **)t1;
-	const char **name2 = (const char **)t2;
+	char const * const * name1 = (char const * const *)t1;
+	char const * const * name2 = (char const * const *)t2;
 
 	return strcmp(*name1, *name2);
 }
@@ -2077,12 +2153,34 @@ host(
 	FILE *fp
 	)
 {
+	int i;
+
 	if (pcmd->nargs == 0) {
 		if (havehost)
 		    (void) fprintf(fp, "current host is %s\n", currenthost);
 		else
 		    (void) fprintf(fp, "no current host\n");
-	} else if (openhost(pcmd->argval[0].string)) {
+		return;
+	}
+
+	i = 0;
+	ai_fam_templ = 0;
+	if (pcmd->nargs == 2) {
+		if (!strcmp("-4", pcmd->argval[i].string))
+			ai_fam_templ = AF_INET;
+		else if (!strcmp("-6", pcmd->argval[i].string))
+			ai_fam_templ = AF_INET6;
+		else {
+			if (havehost)
+				(void) fprintf(fp,
+				    "current host remains %s\n", currenthost);
+			else
+				(void) fprintf(fp, "still no current host\n");
+			return;
+		}
+		i = 1;
+	}
+	if (openhost(pcmd->argval[i].string)) {
 		(void) fprintf(fp, "current host set to %s\n", currenthost);
 		numassoc = 0;
 	} else {
@@ -2402,7 +2500,7 @@ getkeyid(
 	fprintf(stderr, "%s", keyprompt); fflush(stderr);
 	for (p=pbuf; (c = getc(fi))!='\n' && c!=EOF;) {
 		if (p < &pbuf[18])
-		    *p++ = c;
+		    *p++ = (char)c;
 	}
 	*p = '\0';
 	if (fi != stdin)
@@ -2450,7 +2548,7 @@ atoascii(
 
 		if (c < ' ') {
 			*ocp++ = '^';
-			*ocp++ = c + '@';
+			*ocp++ = (u_char)(c + '@');
 		} else if (c == 0177) {
 			*ocp++ = '^';
 			*ocp++ = '?';
@@ -2875,12 +2973,12 @@ cookedprint(
 	register int varid;
 	char *name;
 	char *value;
-	int output_raw;
+	char output_raw;
 	int fmt;
 	struct ctl_var *varlist;
 	l_fp lfp;
 	long ival;
-	u_int32 hval;
+	struct sockaddr_storage hval;
 	u_long uval;
 	l_fp lfparr[8];
 	int narr;
@@ -2959,10 +3057,11 @@ cookedprint(
 			    case NA:
 				if (!decodenetnum(value, &hval))
 				    output_raw = '?';
-				else if (fmt == HA)
-				    output(fp, name, nntohost(hval));
-				else
-				    output(fp, name, numtoa(hval));
+				else if (fmt == HA){
+				    output(fp, name, nntohost(&hval));
+				} else {
+				    output(fp, name, stoa(&hval));
+				}
 				break;
 			
 			    case ST:
@@ -2970,9 +3069,14 @@ cookedprint(
 				break;
 			
 			    case RF:
-				if (decodenetnum(value, &hval))
-				    output(fp, name, nntohost(hval));
-				else if ((int)strlen(value) <= 4)
+				if (decodenetnum(value, &hval)) {
+					if ((hval.ss_family == AF_INET) &&
+					    ISREFCLOCKADR(&hval))
+    						output(fp, name,
+						    refnumtoa(&hval));
+					else
+				    		output(fp, name, stoa(&hval));
+				} else if ((int)strlen(value) <= 4)
 				    output(fp, name, value);
 				else
 				    output_raw = '?';
